// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_DISCARDS_DISCARDS_MOJOM_H_
#define CHROME_BROWSER_UI_WEBUI_DISCARDS_DISCARDS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/ui/webui/discards/discards.mojom-shared.h"
#include "chrome/browser/ui/webui/discards/discards.mojom-forward.h"
#include "chrome/browser/resource_coordinator/lifecycle_unit_state.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace mojom {

class DiscardsDetailsProviderProxy;

template <typename ImplRefTraits>
class DiscardsDetailsProviderStub;

class DiscardsDetailsProviderRequestValidator;
class DiscardsDetailsProviderResponseValidator;


class  DiscardsDetailsProvider
    : public DiscardsDetailsProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = DiscardsDetailsProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = DiscardsDetailsProviderStub<ImplRefTraits>;

  using RequestValidator_ = DiscardsDetailsProviderRequestValidator;
  using ResponseValidator_ = DiscardsDetailsProviderResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetTabDiscardsInfoMinVersion = 0,
    kGetSiteCharacteristicsDatabaseMinVersion = 0,
    kGetSiteCharacteristicsDatabaseSizeMinVersion = 0,
    kSetAutoDiscardableMinVersion = 0,
    kDiscardByIdMinVersion = 0,
    kFreezeByIdMinVersion = 0,
    kLoadByIdMinVersion = 0,
    kDiscardMinVersion = 0,
  };
  virtual ~DiscardsDetailsProvider() {}


  using GetTabDiscardsInfoCallback = base::OnceCallback<void(std::vector<TabDiscardsInfoPtr>)>;
  
  virtual void GetTabDiscardsInfo(GetTabDiscardsInfoCallback callback) = 0;


  using GetSiteCharacteristicsDatabaseCallback = base::OnceCallback<void(SiteCharacteristicsDatabasePtr)>;
  
  virtual void GetSiteCharacteristicsDatabase(const std::vector<std::string>& explicitly_requested_origins, GetSiteCharacteristicsDatabaseCallback callback) = 0;


  using GetSiteCharacteristicsDatabaseSizeCallback = base::OnceCallback<void(SiteCharacteristicsDatabaseSizePtr)>;
  
  virtual void GetSiteCharacteristicsDatabaseSize(GetSiteCharacteristicsDatabaseSizeCallback callback) = 0;


  using SetAutoDiscardableCallback = base::OnceCallback<void()>;
  
  virtual void SetAutoDiscardable(int32_t tab_id, bool is_auto_discardable, SetAutoDiscardableCallback callback) = 0;


  using DiscardByIdCallback = base::OnceCallback<void()>;
  
  virtual void DiscardById(int32_t tab_id, bool urgent, DiscardByIdCallback callback) = 0;

  
  virtual void FreezeById(int32_t tab_id) = 0;

  
  virtual void LoadById(int32_t tab_id) = 0;


  using DiscardCallback = base::OnceCallback<void()>;
  
  virtual void Discard(bool urgent, DiscardCallback callback) = 0;
};

class  DiscardsDetailsProviderProxy
    : public DiscardsDetailsProvider {
 public:
  using InterfaceType = DiscardsDetailsProvider;

  explicit DiscardsDetailsProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetTabDiscardsInfo(GetTabDiscardsInfoCallback callback) final;
  void GetSiteCharacteristicsDatabase(const std::vector<std::string>& explicitly_requested_origins, GetSiteCharacteristicsDatabaseCallback callback) final;
  void GetSiteCharacteristicsDatabaseSize(GetSiteCharacteristicsDatabaseSizeCallback callback) final;
  void SetAutoDiscardable(int32_t tab_id, bool is_auto_discardable, SetAutoDiscardableCallback callback) final;
  void DiscardById(int32_t tab_id, bool urgent, DiscardByIdCallback callback) final;
  void FreezeById(int32_t tab_id) final;
  void LoadById(int32_t tab_id) final;
  void Discard(bool urgent, DiscardCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  DiscardsDetailsProviderStubDispatch {
 public:
  static bool Accept(DiscardsDetailsProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      DiscardsDetailsProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<DiscardsDetailsProvider>>
class DiscardsDetailsProviderStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DiscardsDetailsProviderStub() {}
  ~DiscardsDetailsProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DiscardsDetailsProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DiscardsDetailsProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  DiscardsDetailsProviderRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  DiscardsDetailsProviderResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class  SiteCharacteristicsFeature {
 public:
  using DataView = SiteCharacteristicsFeatureDataView;
  using Data_ = internal::SiteCharacteristicsFeature_Data;

  template <typename... Args>
  static SiteCharacteristicsFeaturePtr New(Args&&... args) {
    return SiteCharacteristicsFeaturePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SiteCharacteristicsFeaturePtr From(const U& u) {
    return mojo::TypeConverter<SiteCharacteristicsFeaturePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SiteCharacteristicsFeature>::Convert(*this);
  }


  SiteCharacteristicsFeature();

  SiteCharacteristicsFeature(
      int64_t observation_duration,
      int64_t use_timestamp);

  ~SiteCharacteristicsFeature();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SiteCharacteristicsFeaturePtr>
  SiteCharacteristicsFeaturePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SiteCharacteristicsFeature>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SiteCharacteristicsFeature::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SiteCharacteristicsFeature::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SiteCharacteristicsFeature_UnserializedMessageContext<
            UserType, SiteCharacteristicsFeature::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SiteCharacteristicsFeature::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SiteCharacteristicsFeature::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SiteCharacteristicsFeature_UnserializedMessageContext<
            UserType, SiteCharacteristicsFeature::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SiteCharacteristicsFeature::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t observation_duration;
  
  int64_t use_timestamp;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  SiteCharacteristicsPerformanceMeasurement {
 public:
  using DataView = SiteCharacteristicsPerformanceMeasurementDataView;
  using Data_ = internal::SiteCharacteristicsPerformanceMeasurement_Data;

  template <typename... Args>
  static SiteCharacteristicsPerformanceMeasurementPtr New(Args&&... args) {
    return SiteCharacteristicsPerformanceMeasurementPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SiteCharacteristicsPerformanceMeasurementPtr From(const U& u) {
    return mojo::TypeConverter<SiteCharacteristicsPerformanceMeasurementPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SiteCharacteristicsPerformanceMeasurement>::Convert(*this);
  }


  SiteCharacteristicsPerformanceMeasurement();

  SiteCharacteristicsPerformanceMeasurement(
      float avg_cpu_usage_us,
      float avg_footprint_kb,
      float avg_load_duration_us);

  ~SiteCharacteristicsPerformanceMeasurement();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SiteCharacteristicsPerformanceMeasurementPtr>
  SiteCharacteristicsPerformanceMeasurementPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SiteCharacteristicsPerformanceMeasurement>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SiteCharacteristicsPerformanceMeasurement::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SiteCharacteristicsPerformanceMeasurement::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SiteCharacteristicsPerformanceMeasurement_UnserializedMessageContext<
            UserType, SiteCharacteristicsPerformanceMeasurement::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SiteCharacteristicsPerformanceMeasurement::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SiteCharacteristicsPerformanceMeasurement::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SiteCharacteristicsPerformanceMeasurement_UnserializedMessageContext<
            UserType, SiteCharacteristicsPerformanceMeasurement::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SiteCharacteristicsPerformanceMeasurement::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  float avg_cpu_usage_us;
  
  float avg_footprint_kb;
  
  float avg_load_duration_us;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  SiteCharacteristicsDatabaseSize {
 public:
  using DataView = SiteCharacteristicsDatabaseSizeDataView;
  using Data_ = internal::SiteCharacteristicsDatabaseSize_Data;

  template <typename... Args>
  static SiteCharacteristicsDatabaseSizePtr New(Args&&... args) {
    return SiteCharacteristicsDatabaseSizePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SiteCharacteristicsDatabaseSizePtr From(const U& u) {
    return mojo::TypeConverter<SiteCharacteristicsDatabaseSizePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SiteCharacteristicsDatabaseSize>::Convert(*this);
  }


  SiteCharacteristicsDatabaseSize();

  SiteCharacteristicsDatabaseSize(
      int64_t num_rows,
      int64_t on_disk_size_kb);

  ~SiteCharacteristicsDatabaseSize();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SiteCharacteristicsDatabaseSizePtr>
  SiteCharacteristicsDatabaseSizePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SiteCharacteristicsDatabaseSize>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SiteCharacteristicsDatabaseSize::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SiteCharacteristicsDatabaseSize::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SiteCharacteristicsDatabaseSize_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabaseSize::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabaseSize::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SiteCharacteristicsDatabaseSize::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SiteCharacteristicsDatabaseSize_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabaseSize::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabaseSize::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int64_t num_rows;
  
  int64_t on_disk_size_kb;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};














class  SiteCharacteristicsDatabaseValue {
 public:
  using DataView = SiteCharacteristicsDatabaseValueDataView;
  using Data_ = internal::SiteCharacteristicsDatabaseValue_Data;

  template <typename... Args>
  static SiteCharacteristicsDatabaseValuePtr New(Args&&... args) {
    return SiteCharacteristicsDatabaseValuePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SiteCharacteristicsDatabaseValuePtr From(const U& u) {
    return mojo::TypeConverter<SiteCharacteristicsDatabaseValuePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SiteCharacteristicsDatabaseValue>::Convert(*this);
  }


  SiteCharacteristicsDatabaseValue();

  SiteCharacteristicsDatabaseValue(
      uint32_t last_loaded,
      SiteCharacteristicsFeaturePtr updates_favicon_in_background,
      SiteCharacteristicsFeaturePtr updates_title_in_background,
      SiteCharacteristicsFeaturePtr uses_audio_in_background,
      SiteCharacteristicsFeaturePtr uses_notifications_in_background,
      SiteCharacteristicsPerformanceMeasurementPtr load_time_estimates);

  ~SiteCharacteristicsDatabaseValue();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SiteCharacteristicsDatabaseValuePtr>
  SiteCharacteristicsDatabaseValuePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SiteCharacteristicsDatabaseValue>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SiteCharacteristicsDatabaseValue::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SiteCharacteristicsDatabaseValue::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SiteCharacteristicsDatabaseValue_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabaseValue::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabaseValue::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SiteCharacteristicsDatabaseValue::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SiteCharacteristicsDatabaseValue_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabaseValue::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabaseValue::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  uint32_t last_loaded;
  
  SiteCharacteristicsFeaturePtr updates_favicon_in_background;
  
  SiteCharacteristicsFeaturePtr updates_title_in_background;
  
  SiteCharacteristicsFeaturePtr uses_audio_in_background;
  
  SiteCharacteristicsFeaturePtr uses_notifications_in_background;
  
  SiteCharacteristicsPerformanceMeasurementPtr load_time_estimates;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(SiteCharacteristicsDatabaseValue);
};





class  SiteCharacteristicsDatabaseEntry {
 public:
  using DataView = SiteCharacteristicsDatabaseEntryDataView;
  using Data_ = internal::SiteCharacteristicsDatabaseEntry_Data;

  template <typename... Args>
  static SiteCharacteristicsDatabaseEntryPtr New(Args&&... args) {
    return SiteCharacteristicsDatabaseEntryPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SiteCharacteristicsDatabaseEntryPtr From(const U& u) {
    return mojo::TypeConverter<SiteCharacteristicsDatabaseEntryPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SiteCharacteristicsDatabaseEntry>::Convert(*this);
  }


  SiteCharacteristicsDatabaseEntry();

  SiteCharacteristicsDatabaseEntry(
      const std::string& origin,
      bool is_dirty,
      SiteCharacteristicsDatabaseValuePtr value);

  ~SiteCharacteristicsDatabaseEntry();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SiteCharacteristicsDatabaseEntryPtr>
  SiteCharacteristicsDatabaseEntryPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SiteCharacteristicsDatabaseEntry>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SiteCharacteristicsDatabaseEntry::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SiteCharacteristicsDatabaseEntry::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SiteCharacteristicsDatabaseEntry_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabaseEntry::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabaseEntry::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SiteCharacteristicsDatabaseEntry::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SiteCharacteristicsDatabaseEntry_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabaseEntry::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabaseEntry::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string origin;
  
  bool is_dirty;
  
  SiteCharacteristicsDatabaseValuePtr value;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(SiteCharacteristicsDatabaseEntry);
};





class  SiteCharacteristicsDatabase {
 public:
  using DataView = SiteCharacteristicsDatabaseDataView;
  using Data_ = internal::SiteCharacteristicsDatabase_Data;

  template <typename... Args>
  static SiteCharacteristicsDatabasePtr New(Args&&... args) {
    return SiteCharacteristicsDatabasePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SiteCharacteristicsDatabasePtr From(const U& u) {
    return mojo::TypeConverter<SiteCharacteristicsDatabasePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SiteCharacteristicsDatabase>::Convert(*this);
  }


  SiteCharacteristicsDatabase();

  explicit SiteCharacteristicsDatabase(
      std::vector<SiteCharacteristicsDatabaseEntryPtr> db_rows);

  ~SiteCharacteristicsDatabase();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SiteCharacteristicsDatabasePtr>
  SiteCharacteristicsDatabasePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SiteCharacteristicsDatabase>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SiteCharacteristicsDatabase::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SiteCharacteristicsDatabase::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SiteCharacteristicsDatabase_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabase::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabase::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SiteCharacteristicsDatabase::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SiteCharacteristicsDatabase_UnserializedMessageContext<
            UserType, SiteCharacteristicsDatabase::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SiteCharacteristicsDatabase::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::vector<SiteCharacteristicsDatabaseEntryPtr> db_rows;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(SiteCharacteristicsDatabase);
};





class  TabDiscardsInfo {
 public:
  using DataView = TabDiscardsInfoDataView;
  using Data_ = internal::TabDiscardsInfo_Data;

  template <typename... Args>
  static TabDiscardsInfoPtr New(Args&&... args) {
    return TabDiscardsInfoPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static TabDiscardsInfoPtr From(const U& u) {
    return mojo::TypeConverter<TabDiscardsInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, TabDiscardsInfo>::Convert(*this);
  }


  TabDiscardsInfo();

  TabDiscardsInfo(
      const std::string& tab_url,
      const std::string& title,
      LifecycleUnitVisibility visibility,
      ::mojom::LifecycleUnitLoadingState loading_state,
      ::mojom::LifecycleUnitState state,
      bool can_freeze,
      const std::vector<std::string>& cannot_freeze_reasons,
      bool can_discard,
      const std::vector<std::string>& cannot_discard_reasons,
      int32_t discard_count,
      ::mojom::LifecycleUnitDiscardReason discard_reason,
      int32_t utility_rank,
      int32_t last_active_seconds,
      int32_t id,
      bool is_auto_discardable,
      bool has_reactivation_score,
      double reactivation_score,
      double site_engagement_score,
      bool has_focus);

  ~TabDiscardsInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = TabDiscardsInfoPtr>
  TabDiscardsInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, TabDiscardsInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        TabDiscardsInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        TabDiscardsInfo::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::TabDiscardsInfo_UnserializedMessageContext<
            UserType, TabDiscardsInfo::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<TabDiscardsInfo::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return TabDiscardsInfo::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::TabDiscardsInfo_UnserializedMessageContext<
            UserType, TabDiscardsInfo::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<TabDiscardsInfo::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string tab_url;
  
  std::string title;
  
  LifecycleUnitVisibility visibility;
  
  ::mojom::LifecycleUnitLoadingState loading_state;
  
  ::mojom::LifecycleUnitState state;
  
  bool can_freeze;
  
  std::vector<std::string> cannot_freeze_reasons;
  
  bool can_discard;
  
  std::vector<std::string> cannot_discard_reasons;
  
  int32_t discard_count;
  
  ::mojom::LifecycleUnitDiscardReason discard_reason;
  
  int32_t utility_rank;
  
  int32_t last_active_seconds;
  
  int32_t id;
  
  bool is_auto_discardable;
  
  bool has_reactivation_score;
  
  double reactivation_score;
  
  double site_engagement_score;
  
  bool has_focus;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
SiteCharacteristicsFeaturePtr SiteCharacteristicsFeature::Clone() const {
  return New(
      mojo::Clone(observation_duration),
      mojo::Clone(use_timestamp)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SiteCharacteristicsFeature>::value>::type*>
bool SiteCharacteristicsFeature::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->observation_duration, other_struct.observation_duration))
    return false;
  if (!mojo::Equals(this->use_timestamp, other_struct.use_timestamp))
    return false;
  return true;
}
template <typename StructPtrType>
SiteCharacteristicsPerformanceMeasurementPtr SiteCharacteristicsPerformanceMeasurement::Clone() const {
  return New(
      mojo::Clone(avg_cpu_usage_us),
      mojo::Clone(avg_footprint_kb),
      mojo::Clone(avg_load_duration_us)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SiteCharacteristicsPerformanceMeasurement>::value>::type*>
bool SiteCharacteristicsPerformanceMeasurement::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->avg_cpu_usage_us, other_struct.avg_cpu_usage_us))
    return false;
  if (!mojo::Equals(this->avg_footprint_kb, other_struct.avg_footprint_kb))
    return false;
  if (!mojo::Equals(this->avg_load_duration_us, other_struct.avg_load_duration_us))
    return false;
  return true;
}
template <typename StructPtrType>
SiteCharacteristicsDatabaseSizePtr SiteCharacteristicsDatabaseSize::Clone() const {
  return New(
      mojo::Clone(num_rows),
      mojo::Clone(on_disk_size_kb)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SiteCharacteristicsDatabaseSize>::value>::type*>
bool SiteCharacteristicsDatabaseSize::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->num_rows, other_struct.num_rows))
    return false;
  if (!mojo::Equals(this->on_disk_size_kb, other_struct.on_disk_size_kb))
    return false;
  return true;
}
template <typename StructPtrType>
SiteCharacteristicsDatabaseValuePtr SiteCharacteristicsDatabaseValue::Clone() const {
  return New(
      mojo::Clone(last_loaded),
      mojo::Clone(updates_favicon_in_background),
      mojo::Clone(updates_title_in_background),
      mojo::Clone(uses_audio_in_background),
      mojo::Clone(uses_notifications_in_background),
      mojo::Clone(load_time_estimates)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SiteCharacteristicsDatabaseValue>::value>::type*>
bool SiteCharacteristicsDatabaseValue::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->last_loaded, other_struct.last_loaded))
    return false;
  if (!mojo::Equals(this->updates_favicon_in_background, other_struct.updates_favicon_in_background))
    return false;
  if (!mojo::Equals(this->updates_title_in_background, other_struct.updates_title_in_background))
    return false;
  if (!mojo::Equals(this->uses_audio_in_background, other_struct.uses_audio_in_background))
    return false;
  if (!mojo::Equals(this->uses_notifications_in_background, other_struct.uses_notifications_in_background))
    return false;
  if (!mojo::Equals(this->load_time_estimates, other_struct.load_time_estimates))
    return false;
  return true;
}
template <typename StructPtrType>
SiteCharacteristicsDatabaseEntryPtr SiteCharacteristicsDatabaseEntry::Clone() const {
  return New(
      mojo::Clone(origin),
      mojo::Clone(is_dirty),
      mojo::Clone(value)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SiteCharacteristicsDatabaseEntry>::value>::type*>
bool SiteCharacteristicsDatabaseEntry::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->origin, other_struct.origin))
    return false;
  if (!mojo::Equals(this->is_dirty, other_struct.is_dirty))
    return false;
  if (!mojo::Equals(this->value, other_struct.value))
    return false;
  return true;
}
template <typename StructPtrType>
SiteCharacteristicsDatabasePtr SiteCharacteristicsDatabase::Clone() const {
  return New(
      mojo::Clone(db_rows)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SiteCharacteristicsDatabase>::value>::type*>
bool SiteCharacteristicsDatabase::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->db_rows, other_struct.db_rows))
    return false;
  return true;
}
template <typename StructPtrType>
TabDiscardsInfoPtr TabDiscardsInfo::Clone() const {
  return New(
      mojo::Clone(tab_url),
      mojo::Clone(title),
      mojo::Clone(visibility),
      mojo::Clone(loading_state),
      mojo::Clone(state),
      mojo::Clone(can_freeze),
      mojo::Clone(cannot_freeze_reasons),
      mojo::Clone(can_discard),
      mojo::Clone(cannot_discard_reasons),
      mojo::Clone(discard_count),
      mojo::Clone(discard_reason),
      mojo::Clone(utility_rank),
      mojo::Clone(last_active_seconds),
      mojo::Clone(id),
      mojo::Clone(is_auto_discardable),
      mojo::Clone(has_reactivation_score),
      mojo::Clone(reactivation_score),
      mojo::Clone(site_engagement_score),
      mojo::Clone(has_focus)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, TabDiscardsInfo>::value>::type*>
bool TabDiscardsInfo::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->tab_url, other_struct.tab_url))
    return false;
  if (!mojo::Equals(this->title, other_struct.title))
    return false;
  if (!mojo::Equals(this->visibility, other_struct.visibility))
    return false;
  if (!mojo::Equals(this->loading_state, other_struct.loading_state))
    return false;
  if (!mojo::Equals(this->state, other_struct.state))
    return false;
  if (!mojo::Equals(this->can_freeze, other_struct.can_freeze))
    return false;
  if (!mojo::Equals(this->cannot_freeze_reasons, other_struct.cannot_freeze_reasons))
    return false;
  if (!mojo::Equals(this->can_discard, other_struct.can_discard))
    return false;
  if (!mojo::Equals(this->cannot_discard_reasons, other_struct.cannot_discard_reasons))
    return false;
  if (!mojo::Equals(this->discard_count, other_struct.discard_count))
    return false;
  if (!mojo::Equals(this->discard_reason, other_struct.discard_reason))
    return false;
  if (!mojo::Equals(this->utility_rank, other_struct.utility_rank))
    return false;
  if (!mojo::Equals(this->last_active_seconds, other_struct.last_active_seconds))
    return false;
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->is_auto_discardable, other_struct.is_auto_discardable))
    return false;
  if (!mojo::Equals(this->has_reactivation_score, other_struct.has_reactivation_score))
    return false;
  if (!mojo::Equals(this->reactivation_score, other_struct.reactivation_score))
    return false;
  if (!mojo::Equals(this->site_engagement_score, other_struct.site_engagement_score))
    return false;
  if (!mojo::Equals(this->has_focus, other_struct.has_focus))
    return false;
  return true;
}


}  // namespace mojom

namespace mojo {


template <>
struct  StructTraits<::mojom::SiteCharacteristicsFeature::DataView,
                                         ::mojom::SiteCharacteristicsFeaturePtr> {
  static bool IsNull(const ::mojom::SiteCharacteristicsFeaturePtr& input) { return !input; }
  static void SetToNull(::mojom::SiteCharacteristicsFeaturePtr* output) { output->reset(); }

  static decltype(::mojom::SiteCharacteristicsFeature::observation_duration) observation_duration(
      const ::mojom::SiteCharacteristicsFeaturePtr& input) {
    return input->observation_duration;
  }

  static decltype(::mojom::SiteCharacteristicsFeature::use_timestamp) use_timestamp(
      const ::mojom::SiteCharacteristicsFeaturePtr& input) {
    return input->use_timestamp;
  }

  static bool Read(::mojom::SiteCharacteristicsFeature::DataView input, ::mojom::SiteCharacteristicsFeaturePtr* output);
};


template <>
struct  StructTraits<::mojom::SiteCharacteristicsPerformanceMeasurement::DataView,
                                         ::mojom::SiteCharacteristicsPerformanceMeasurementPtr> {
  static bool IsNull(const ::mojom::SiteCharacteristicsPerformanceMeasurementPtr& input) { return !input; }
  static void SetToNull(::mojom::SiteCharacteristicsPerformanceMeasurementPtr* output) { output->reset(); }

  static decltype(::mojom::SiteCharacteristicsPerformanceMeasurement::avg_cpu_usage_us) avg_cpu_usage_us(
      const ::mojom::SiteCharacteristicsPerformanceMeasurementPtr& input) {
    return input->avg_cpu_usage_us;
  }

  static decltype(::mojom::SiteCharacteristicsPerformanceMeasurement::avg_footprint_kb) avg_footprint_kb(
      const ::mojom::SiteCharacteristicsPerformanceMeasurementPtr& input) {
    return input->avg_footprint_kb;
  }

  static decltype(::mojom::SiteCharacteristicsPerformanceMeasurement::avg_load_duration_us) avg_load_duration_us(
      const ::mojom::SiteCharacteristicsPerformanceMeasurementPtr& input) {
    return input->avg_load_duration_us;
  }

  static bool Read(::mojom::SiteCharacteristicsPerformanceMeasurement::DataView input, ::mojom::SiteCharacteristicsPerformanceMeasurementPtr* output);
};


template <>
struct  StructTraits<::mojom::SiteCharacteristicsDatabaseSize::DataView,
                                         ::mojom::SiteCharacteristicsDatabaseSizePtr> {
  static bool IsNull(const ::mojom::SiteCharacteristicsDatabaseSizePtr& input) { return !input; }
  static void SetToNull(::mojom::SiteCharacteristicsDatabaseSizePtr* output) { output->reset(); }

  static decltype(::mojom::SiteCharacteristicsDatabaseSize::num_rows) num_rows(
      const ::mojom::SiteCharacteristicsDatabaseSizePtr& input) {
    return input->num_rows;
  }

  static decltype(::mojom::SiteCharacteristicsDatabaseSize::on_disk_size_kb) on_disk_size_kb(
      const ::mojom::SiteCharacteristicsDatabaseSizePtr& input) {
    return input->on_disk_size_kb;
  }

  static bool Read(::mojom::SiteCharacteristicsDatabaseSize::DataView input, ::mojom::SiteCharacteristicsDatabaseSizePtr* output);
};


template <>
struct  StructTraits<::mojom::SiteCharacteristicsDatabaseValue::DataView,
                                         ::mojom::SiteCharacteristicsDatabaseValuePtr> {
  static bool IsNull(const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) { return !input; }
  static void SetToNull(::mojom::SiteCharacteristicsDatabaseValuePtr* output) { output->reset(); }

  static decltype(::mojom::SiteCharacteristicsDatabaseValue::last_loaded) last_loaded(
      const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) {
    return input->last_loaded;
  }

  static const decltype(::mojom::SiteCharacteristicsDatabaseValue::updates_favicon_in_background)& updates_favicon_in_background(
      const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) {
    return input->updates_favicon_in_background;
  }

  static const decltype(::mojom::SiteCharacteristicsDatabaseValue::updates_title_in_background)& updates_title_in_background(
      const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) {
    return input->updates_title_in_background;
  }

  static const decltype(::mojom::SiteCharacteristicsDatabaseValue::uses_audio_in_background)& uses_audio_in_background(
      const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) {
    return input->uses_audio_in_background;
  }

  static const decltype(::mojom::SiteCharacteristicsDatabaseValue::uses_notifications_in_background)& uses_notifications_in_background(
      const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) {
    return input->uses_notifications_in_background;
  }

  static const decltype(::mojom::SiteCharacteristicsDatabaseValue::load_time_estimates)& load_time_estimates(
      const ::mojom::SiteCharacteristicsDatabaseValuePtr& input) {
    return input->load_time_estimates;
  }

  static bool Read(::mojom::SiteCharacteristicsDatabaseValue::DataView input, ::mojom::SiteCharacteristicsDatabaseValuePtr* output);
};


template <>
struct  StructTraits<::mojom::SiteCharacteristicsDatabaseEntry::DataView,
                                         ::mojom::SiteCharacteristicsDatabaseEntryPtr> {
  static bool IsNull(const ::mojom::SiteCharacteristicsDatabaseEntryPtr& input) { return !input; }
  static void SetToNull(::mojom::SiteCharacteristicsDatabaseEntryPtr* output) { output->reset(); }

  static const decltype(::mojom::SiteCharacteristicsDatabaseEntry::origin)& origin(
      const ::mojom::SiteCharacteristicsDatabaseEntryPtr& input) {
    return input->origin;
  }

  static decltype(::mojom::SiteCharacteristicsDatabaseEntry::is_dirty) is_dirty(
      const ::mojom::SiteCharacteristicsDatabaseEntryPtr& input) {
    return input->is_dirty;
  }

  static const decltype(::mojom::SiteCharacteristicsDatabaseEntry::value)& value(
      const ::mojom::SiteCharacteristicsDatabaseEntryPtr& input) {
    return input->value;
  }

  static bool Read(::mojom::SiteCharacteristicsDatabaseEntry::DataView input, ::mojom::SiteCharacteristicsDatabaseEntryPtr* output);
};


template <>
struct  StructTraits<::mojom::SiteCharacteristicsDatabase::DataView,
                                         ::mojom::SiteCharacteristicsDatabasePtr> {
  static bool IsNull(const ::mojom::SiteCharacteristicsDatabasePtr& input) { return !input; }
  static void SetToNull(::mojom::SiteCharacteristicsDatabasePtr* output) { output->reset(); }

  static const decltype(::mojom::SiteCharacteristicsDatabase::db_rows)& db_rows(
      const ::mojom::SiteCharacteristicsDatabasePtr& input) {
    return input->db_rows;
  }

  static bool Read(::mojom::SiteCharacteristicsDatabase::DataView input, ::mojom::SiteCharacteristicsDatabasePtr* output);
};


template <>
struct  StructTraits<::mojom::TabDiscardsInfo::DataView,
                                         ::mojom::TabDiscardsInfoPtr> {
  static bool IsNull(const ::mojom::TabDiscardsInfoPtr& input) { return !input; }
  static void SetToNull(::mojom::TabDiscardsInfoPtr* output) { output->reset(); }

  static const decltype(::mojom::TabDiscardsInfo::tab_url)& tab_url(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->tab_url;
  }

  static const decltype(::mojom::TabDiscardsInfo::title)& title(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->title;
  }

  static decltype(::mojom::TabDiscardsInfo::visibility) visibility(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->visibility;
  }

  static decltype(::mojom::TabDiscardsInfo::loading_state) loading_state(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->loading_state;
  }

  static decltype(::mojom::TabDiscardsInfo::state) state(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->state;
  }

  static decltype(::mojom::TabDiscardsInfo::can_freeze) can_freeze(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->can_freeze;
  }

  static const decltype(::mojom::TabDiscardsInfo::cannot_freeze_reasons)& cannot_freeze_reasons(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->cannot_freeze_reasons;
  }

  static decltype(::mojom::TabDiscardsInfo::can_discard) can_discard(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->can_discard;
  }

  static const decltype(::mojom::TabDiscardsInfo::cannot_discard_reasons)& cannot_discard_reasons(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->cannot_discard_reasons;
  }

  static decltype(::mojom::TabDiscardsInfo::discard_count) discard_count(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->discard_count;
  }

  static decltype(::mojom::TabDiscardsInfo::discard_reason) discard_reason(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->discard_reason;
  }

  static decltype(::mojom::TabDiscardsInfo::utility_rank) utility_rank(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->utility_rank;
  }

  static decltype(::mojom::TabDiscardsInfo::last_active_seconds) last_active_seconds(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->last_active_seconds;
  }

  static decltype(::mojom::TabDiscardsInfo::id) id(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->id;
  }

  static decltype(::mojom::TabDiscardsInfo::is_auto_discardable) is_auto_discardable(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->is_auto_discardable;
  }

  static decltype(::mojom::TabDiscardsInfo::has_reactivation_score) has_reactivation_score(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->has_reactivation_score;
  }

  static decltype(::mojom::TabDiscardsInfo::reactivation_score) reactivation_score(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->reactivation_score;
  }

  static decltype(::mojom::TabDiscardsInfo::site_engagement_score) site_engagement_score(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->site_engagement_score;
  }

  static decltype(::mojom::TabDiscardsInfo::has_focus) has_focus(
      const ::mojom::TabDiscardsInfoPtr& input) {
    return input->has_focus;
  }

  static bool Read(::mojom::TabDiscardsInfo::DataView input, ::mojom::TabDiscardsInfoPtr* output);
};

}  // namespace mojo

#endif  // CHROME_BROWSER_UI_WEBUI_DISCARDS_DISCARDS_MOJOM_H_