// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_BROWSER_UI_WEBUI_FEED_INTERNALS_FEED_INTERNALS_MOJOM_H_
#define CHROME_BROWSER_UI_WEBUI_FEED_INTERNALS_FEED_INTERNALS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/browser/ui/webui/feed_internals/feed_internals.mojom-shared.h"
#include "chrome/browser/ui/webui/feed_internals/feed_internals.mojom-forward.h"
#include "url/mojom/url.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace feed_internals {
namespace mojom {

class PageHandlerProxy;

template <typename ImplRefTraits>
class PageHandlerStub;

class PageHandlerRequestValidator;
class PageHandlerResponseValidator;


class  PageHandler
    : public PageHandlerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = PageHandlerProxy;

  template <typename ImplRefTraits>
  using Stub_ = PageHandlerStub<ImplRefTraits>;

  using RequestValidator_ = PageHandlerRequestValidator;
  using ResponseValidator_ = PageHandlerResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetGeneralPropertiesMinVersion = 0,
    kGetUserClassifierPropertiesMinVersion = 0,
    kClearUserClassifierPropertiesMinVersion = 0,
    kGetLastFetchPropertiesMinVersion = 0,
    kClearCachedDataAndRefreshFeedMinVersion = 0,
    kRefreshFeedMinVersion = 0,
    kGetCurrentContentMinVersion = 0,
    kGetFeedProcessScopeDumpMinVersion = 0,
    kGetFeedHistogramsMinVersion = 0,
  };
  virtual ~PageHandler() {}


  using GetGeneralPropertiesCallback = base::OnceCallback<void(PropertiesPtr)>;
  
  virtual void GetGeneralProperties(GetGeneralPropertiesCallback callback) = 0;


  using GetUserClassifierPropertiesCallback = base::OnceCallback<void(UserClassifierPtr)>;
  
  virtual void GetUserClassifierProperties(GetUserClassifierPropertiesCallback callback) = 0;

  
  virtual void ClearUserClassifierProperties() = 0;


  using GetLastFetchPropertiesCallback = base::OnceCallback<void(LastFetchPropertiesPtr)>;
  
  virtual void GetLastFetchProperties(GetLastFetchPropertiesCallback callback) = 0;

  
  virtual void ClearCachedDataAndRefreshFeed() = 0;

  
  virtual void RefreshFeed() = 0;


  using GetCurrentContentCallback = base::OnceCallback<void(std::vector<SuggestionPtr>)>;
  
  virtual void GetCurrentContent(GetCurrentContentCallback callback) = 0;


  using GetFeedProcessScopeDumpCallback = base::OnceCallback<void(const std::string&)>;
  
  virtual void GetFeedProcessScopeDump(GetFeedProcessScopeDumpCallback callback) = 0;


  using GetFeedHistogramsCallback = base::OnceCallback<void(const std::string&)>;
  
  virtual void GetFeedHistograms(GetFeedHistogramsCallback callback) = 0;
};

class  PageHandlerProxy
    : public PageHandler {
 public:
  using InterfaceType = PageHandler;

  explicit PageHandlerProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetGeneralProperties(GetGeneralPropertiesCallback callback) final;
  void GetUserClassifierProperties(GetUserClassifierPropertiesCallback callback) final;
  void ClearUserClassifierProperties() final;
  void GetLastFetchProperties(GetLastFetchPropertiesCallback callback) final;
  void ClearCachedDataAndRefreshFeed() final;
  void RefreshFeed() final;
  void GetCurrentContent(GetCurrentContentCallback callback) final;
  void GetFeedProcessScopeDump(GetFeedProcessScopeDumpCallback callback) final;
  void GetFeedHistograms(GetFeedHistogramsCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  PageHandlerStubDispatch {
 public:
  static bool Accept(PageHandler* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      PageHandler* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<PageHandler>>
class PageHandlerStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  PageHandlerStub() {}
  ~PageHandlerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return PageHandlerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  PageHandlerRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  PageHandlerResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};






class  UserClassifier {
 public:
  using DataView = UserClassifierDataView;
  using Data_ = internal::UserClassifier_Data;

  template <typename... Args>
  static UserClassifierPtr New(Args&&... args) {
    return UserClassifierPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static UserClassifierPtr From(const U& u) {
    return mojo::TypeConverter<UserClassifierPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, UserClassifier>::Convert(*this);
  }


  UserClassifier();

  UserClassifier(
      const std::string& user_class_description,
      float avg_hours_between_views,
      float avg_hours_between_uses);

  ~UserClassifier();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = UserClassifierPtr>
  UserClassifierPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, UserClassifier>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        UserClassifier::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        UserClassifier::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::UserClassifier_UnserializedMessageContext<
            UserType, UserClassifier::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<UserClassifier::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return UserClassifier::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::UserClassifier_UnserializedMessageContext<
            UserType, UserClassifier::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<UserClassifier::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string user_class_description;
  
  float avg_hours_between_views;
  
  float avg_hours_between_uses;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class  Time {
 public:
  using DataView = TimeDataView;
  using Data_ = internal::Time_Data;

  template <typename... Args>
  static TimePtr New(Args&&... args) {
    return TimePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static TimePtr From(const U& u) {
    return mojo::TypeConverter<TimePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Time>::Convert(*this);
  }


  Time();

  explicit Time(
      double ms_since_epoch);

  ~Time();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = TimePtr>
  TimePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Time>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        Time::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        Time::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::Time_UnserializedMessageContext<
            UserType, Time::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<Time::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return Time::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::Time_UnserializedMessageContext<
            UserType, Time::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<Time::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  double ms_since_epoch;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class  Properties {
 public:
  using DataView = PropertiesDataView;
  using Data_ = internal::Properties_Data;

  template <typename... Args>
  static PropertiesPtr New(Args&&... args) {
    return PropertiesPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static PropertiesPtr From(const U& u) {
    return mojo::TypeConverter<PropertiesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Properties>::Convert(*this);
  }


  Properties();

  Properties(
      bool is_feed_enabled,
      bool is_feed_visible,
      bool is_feed_allowed,
      bool is_prefetching_enabled,
      const GURL& feed_fetch_url);

  ~Properties();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PropertiesPtr>
  PropertiesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Properties>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        Properties::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        Properties::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::Properties_UnserializedMessageContext<
            UserType, Properties::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<Properties::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return Properties::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::Properties_UnserializedMessageContext<
            UserType, Properties::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<Properties::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  bool is_feed_enabled;
  
  bool is_feed_visible;
  
  bool is_feed_allowed;
  
  bool is_prefetching_enabled;
  
  GURL feed_fetch_url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};






class  LastFetchProperties {
 public:
  using DataView = LastFetchPropertiesDataView;
  using Data_ = internal::LastFetchProperties_Data;

  template <typename... Args>
  static LastFetchPropertiesPtr New(Args&&... args) {
    return LastFetchPropertiesPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static LastFetchPropertiesPtr From(const U& u) {
    return mojo::TypeConverter<LastFetchPropertiesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, LastFetchProperties>::Convert(*this);
  }


  LastFetchProperties();

  LastFetchProperties(
      int32_t last_fetch_status,
      const std::string& last_fetch_trigger,
      TimePtr last_fetch_time,
      TimePtr refresh_suppress_time);

  ~LastFetchProperties();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = LastFetchPropertiesPtr>
  LastFetchPropertiesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, LastFetchProperties>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        LastFetchProperties::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        LastFetchProperties::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::LastFetchProperties_UnserializedMessageContext<
            UserType, LastFetchProperties::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<LastFetchProperties::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return LastFetchProperties::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::LastFetchProperties_UnserializedMessageContext<
            UserType, LastFetchProperties::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<LastFetchProperties::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  int32_t last_fetch_status;
  
  std::string last_fetch_trigger;
  
  TimePtr last_fetch_time;
  
  TimePtr refresh_suppress_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(LastFetchProperties);
};





class  Suggestion {
 public:
  using DataView = SuggestionDataView;
  using Data_ = internal::Suggestion_Data;

  template <typename... Args>
  static SuggestionPtr New(Args&&... args) {
    return SuggestionPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SuggestionPtr From(const U& u) {
    return mojo::TypeConverter<SuggestionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Suggestion>::Convert(*this);
  }


  Suggestion();

  Suggestion(
      const std::string& title,
      const GURL& url,
      const std::string& publisher_name,
      const GURL& image_url,
      const GURL& favicon_url);

  ~Suggestion();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SuggestionPtr>
  SuggestionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Suggestion>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        Suggestion::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        Suggestion::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::Suggestion_UnserializedMessageContext<
            UserType, Suggestion::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<Suggestion::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return Suggestion::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::Suggestion_UnserializedMessageContext<
            UserType, Suggestion::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<Suggestion::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string title;
  
  GURL url;
  
  std::string publisher_name;
  
  GURL image_url;
  
  GURL favicon_url;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
PropertiesPtr Properties::Clone() const {
  return New(
      mojo::Clone(is_feed_enabled),
      mojo::Clone(is_feed_visible),
      mojo::Clone(is_feed_allowed),
      mojo::Clone(is_prefetching_enabled),
      mojo::Clone(feed_fetch_url)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Properties>::value>::type*>
bool Properties::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->is_feed_enabled, other_struct.is_feed_enabled))
    return false;
  if (!mojo::Equals(this->is_feed_visible, other_struct.is_feed_visible))
    return false;
  if (!mojo::Equals(this->is_feed_allowed, other_struct.is_feed_allowed))
    return false;
  if (!mojo::Equals(this->is_prefetching_enabled, other_struct.is_prefetching_enabled))
    return false;
  if (!mojo::Equals(this->feed_fetch_url, other_struct.feed_fetch_url))
    return false;
  return true;
}
template <typename StructPtrType>
UserClassifierPtr UserClassifier::Clone() const {
  return New(
      mojo::Clone(user_class_description),
      mojo::Clone(avg_hours_between_views),
      mojo::Clone(avg_hours_between_uses)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, UserClassifier>::value>::type*>
bool UserClassifier::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->user_class_description, other_struct.user_class_description))
    return false;
  if (!mojo::Equals(this->avg_hours_between_views, other_struct.avg_hours_between_views))
    return false;
  if (!mojo::Equals(this->avg_hours_between_uses, other_struct.avg_hours_between_uses))
    return false;
  return true;
}
template <typename StructPtrType>
LastFetchPropertiesPtr LastFetchProperties::Clone() const {
  return New(
      mojo::Clone(last_fetch_status),
      mojo::Clone(last_fetch_trigger),
      mojo::Clone(last_fetch_time),
      mojo::Clone(refresh_suppress_time)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, LastFetchProperties>::value>::type*>
bool LastFetchProperties::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->last_fetch_status, other_struct.last_fetch_status))
    return false;
  if (!mojo::Equals(this->last_fetch_trigger, other_struct.last_fetch_trigger))
    return false;
  if (!mojo::Equals(this->last_fetch_time, other_struct.last_fetch_time))
    return false;
  if (!mojo::Equals(this->refresh_suppress_time, other_struct.refresh_suppress_time))
    return false;
  return true;
}
template <typename StructPtrType>
SuggestionPtr Suggestion::Clone() const {
  return New(
      mojo::Clone(title),
      mojo::Clone(url),
      mojo::Clone(publisher_name),
      mojo::Clone(image_url),
      mojo::Clone(favicon_url)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Suggestion>::value>::type*>
bool Suggestion::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->title, other_struct.title))
    return false;
  if (!mojo::Equals(this->url, other_struct.url))
    return false;
  if (!mojo::Equals(this->publisher_name, other_struct.publisher_name))
    return false;
  if (!mojo::Equals(this->image_url, other_struct.image_url))
    return false;
  if (!mojo::Equals(this->favicon_url, other_struct.favicon_url))
    return false;
  return true;
}
template <typename StructPtrType>
TimePtr Time::Clone() const {
  return New(
      mojo::Clone(ms_since_epoch)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Time>::value>::type*>
bool Time::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->ms_since_epoch, other_struct.ms_since_epoch))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace feed_internals

namespace mojo {


template <>
struct  StructTraits<::feed_internals::mojom::Properties::DataView,
                                         ::feed_internals::mojom::PropertiesPtr> {
  static bool IsNull(const ::feed_internals::mojom::PropertiesPtr& input) { return !input; }
  static void SetToNull(::feed_internals::mojom::PropertiesPtr* output) { output->reset(); }

  static decltype(::feed_internals::mojom::Properties::is_feed_enabled) is_feed_enabled(
      const ::feed_internals::mojom::PropertiesPtr& input) {
    return input->is_feed_enabled;
  }

  static decltype(::feed_internals::mojom::Properties::is_feed_visible) is_feed_visible(
      const ::feed_internals::mojom::PropertiesPtr& input) {
    return input->is_feed_visible;
  }

  static decltype(::feed_internals::mojom::Properties::is_feed_allowed) is_feed_allowed(
      const ::feed_internals::mojom::PropertiesPtr& input) {
    return input->is_feed_allowed;
  }

  static decltype(::feed_internals::mojom::Properties::is_prefetching_enabled) is_prefetching_enabled(
      const ::feed_internals::mojom::PropertiesPtr& input) {
    return input->is_prefetching_enabled;
  }

  static const decltype(::feed_internals::mojom::Properties::feed_fetch_url)& feed_fetch_url(
      const ::feed_internals::mojom::PropertiesPtr& input) {
    return input->feed_fetch_url;
  }

  static bool Read(::feed_internals::mojom::Properties::DataView input, ::feed_internals::mojom::PropertiesPtr* output);
};


template <>
struct  StructTraits<::feed_internals::mojom::UserClassifier::DataView,
                                         ::feed_internals::mojom::UserClassifierPtr> {
  static bool IsNull(const ::feed_internals::mojom::UserClassifierPtr& input) { return !input; }
  static void SetToNull(::feed_internals::mojom::UserClassifierPtr* output) { output->reset(); }

  static const decltype(::feed_internals::mojom::UserClassifier::user_class_description)& user_class_description(
      const ::feed_internals::mojom::UserClassifierPtr& input) {
    return input->user_class_description;
  }

  static decltype(::feed_internals::mojom::UserClassifier::avg_hours_between_views) avg_hours_between_views(
      const ::feed_internals::mojom::UserClassifierPtr& input) {
    return input->avg_hours_between_views;
  }

  static decltype(::feed_internals::mojom::UserClassifier::avg_hours_between_uses) avg_hours_between_uses(
      const ::feed_internals::mojom::UserClassifierPtr& input) {
    return input->avg_hours_between_uses;
  }

  static bool Read(::feed_internals::mojom::UserClassifier::DataView input, ::feed_internals::mojom::UserClassifierPtr* output);
};


template <>
struct  StructTraits<::feed_internals::mojom::LastFetchProperties::DataView,
                                         ::feed_internals::mojom::LastFetchPropertiesPtr> {
  static bool IsNull(const ::feed_internals::mojom::LastFetchPropertiesPtr& input) { return !input; }
  static void SetToNull(::feed_internals::mojom::LastFetchPropertiesPtr* output) { output->reset(); }

  static decltype(::feed_internals::mojom::LastFetchProperties::last_fetch_status) last_fetch_status(
      const ::feed_internals::mojom::LastFetchPropertiesPtr& input) {
    return input->last_fetch_status;
  }

  static const decltype(::feed_internals::mojom::LastFetchProperties::last_fetch_trigger)& last_fetch_trigger(
      const ::feed_internals::mojom::LastFetchPropertiesPtr& input) {
    return input->last_fetch_trigger;
  }

  static const decltype(::feed_internals::mojom::LastFetchProperties::last_fetch_time)& last_fetch_time(
      const ::feed_internals::mojom::LastFetchPropertiesPtr& input) {
    return input->last_fetch_time;
  }

  static const decltype(::feed_internals::mojom::LastFetchProperties::refresh_suppress_time)& refresh_suppress_time(
      const ::feed_internals::mojom::LastFetchPropertiesPtr& input) {
    return input->refresh_suppress_time;
  }

  static bool Read(::feed_internals::mojom::LastFetchProperties::DataView input, ::feed_internals::mojom::LastFetchPropertiesPtr* output);
};


template <>
struct  StructTraits<::feed_internals::mojom::Suggestion::DataView,
                                         ::feed_internals::mojom::SuggestionPtr> {
  static bool IsNull(const ::feed_internals::mojom::SuggestionPtr& input) { return !input; }
  static void SetToNull(::feed_internals::mojom::SuggestionPtr* output) { output->reset(); }

  static const decltype(::feed_internals::mojom::Suggestion::title)& title(
      const ::feed_internals::mojom::SuggestionPtr& input) {
    return input->title;
  }

  static const decltype(::feed_internals::mojom::Suggestion::url)& url(
      const ::feed_internals::mojom::SuggestionPtr& input) {
    return input->url;
  }

  static const decltype(::feed_internals::mojom::Suggestion::publisher_name)& publisher_name(
      const ::feed_internals::mojom::SuggestionPtr& input) {
    return input->publisher_name;
  }

  static const decltype(::feed_internals::mojom::Suggestion::image_url)& image_url(
      const ::feed_internals::mojom::SuggestionPtr& input) {
    return input->image_url;
  }

  static const decltype(::feed_internals::mojom::Suggestion::favicon_url)& favicon_url(
      const ::feed_internals::mojom::SuggestionPtr& input) {
    return input->favicon_url;
  }

  static bool Read(::feed_internals::mojom::Suggestion::DataView input, ::feed_internals::mojom::SuggestionPtr* output);
};


template <>
struct  StructTraits<::feed_internals::mojom::Time::DataView,
                                         ::feed_internals::mojom::TimePtr> {
  static bool IsNull(const ::feed_internals::mojom::TimePtr& input) { return !input; }
  static void SetToNull(::feed_internals::mojom::TimePtr* output) { output->reset(); }

  static decltype(::feed_internals::mojom::Time::ms_since_epoch) ms_since_epoch(
      const ::feed_internals::mojom::TimePtr& input) {
    return input->ms_since_epoch;
  }

  static bool Read(::feed_internals::mojom::Time::DataView input, ::feed_internals::mojom::TimePtr* output);
};

}  // namespace mojo

#endif  // CHROME_BROWSER_UI_WEBUI_FEED_INTERNALS_FEED_INTERNALS_MOJOM_H_