// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_COMMON_SEARCH_MOJOM_TEST_UTILS_H_
#define CHROME_COMMON_SEARCH_MOJOM_TEST_UTILS_H_

#include "chrome/common/search.mojom.h"


namespace chrome {
namespace mojom {


class  EmbeddedSearchConnectorInterceptorForTesting : public EmbeddedSearchConnector {
  virtual EmbeddedSearchConnector* GetForwardingInterface() = 0;
  void Connect(EmbeddedSearchAssociatedRequest embedded_search, EmbeddedSearchClientAssociatedPtrInfo client) override;
};
class  EmbeddedSearchConnectorAsyncWaiter {
 public:
  explicit EmbeddedSearchConnectorAsyncWaiter(EmbeddedSearchConnector* proxy);
  ~EmbeddedSearchConnectorAsyncWaiter();

 private:
  EmbeddedSearchConnector* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(EmbeddedSearchConnectorAsyncWaiter);
};


class  EmbeddedSearchInterceptorForTesting : public EmbeddedSearch {
  virtual EmbeddedSearch* GetForwardingInterface() = 0;
  void FocusOmnibox(int32_t page_seq_no, bool focus) override;
  void DeleteMostVisitedItem(int32_t page_seq_no, const GURL& url) override;
  void UndoAllMostVisitedDeletions(int32_t page_seq_no) override;
  void UndoMostVisitedDeletion(int32_t page_seq_no, const GURL& url) override;
  void ToggleMostVisitedOrCustomLinks(int32_t page_seq_no) override;
  void AddCustomLink(int32_t page_seq_no, const GURL& url, const std::string& title, AddCustomLinkCallback callback) override;
  void UpdateCustomLink(int32_t page_seq_no, const GURL& url, const GURL& new_url, const std::string& new_title, UpdateCustomLinkCallback callback) override;
  void ReorderCustomLink(int32_t page_seq_no, const GURL& url, int32_t new_pos) override;
  void DeleteCustomLink(int32_t page_seq_no, const GURL& url, DeleteCustomLinkCallback callback) override;
  void UndoCustomLinkAction(int32_t page_seq_no) override;
  void ResetCustomLinks(int32_t page_seq_no) override;
  void LogEvent(int32_t page_seq_no, ::NTPLoggingEventType event, base::TimeDelta time) override;
  void LogSuggestionEventWithValue(int32_t page_seq_no, ::NTPSuggestionsLoggingEventType event, int32_t data, base::TimeDelta time) override;
  void LogMostVisitedImpression(int32_t page_seq_no, const ::ntp_tiles::NTPTileImpression& impression) override;
  void LogMostVisitedNavigation(int32_t page_seq_no, const ::ntp_tiles::NTPTileImpression& impression) override;
  void PasteAndOpenDropdown(int32_t page_seq_no, const base::string16& text_to_be_pasted) override;
  void HistorySyncCheck(int32_t page_seq_no, HistorySyncCheckCallback callback) override;
  void ChromeIdentityCheck(int32_t page_seq_no, const base::string16& identity, ChromeIdentityCheckCallback callback) override;
  void SetCustomBackgroundURL(const GURL& url) override;
  void SetCustomBackgroundURLWithAttributions(const GURL& background_url, const std::string& attribution_line_1, const std::string& attribution_line_2, const GURL& action_url) override;
  void SelectLocalBackgroundImage() override;
  void BlocklistSearchSuggestion(int32_t task_version, int64_t task_id) override;
  void BlocklistSearchSuggestionWithHash(int32_t task_version, int64_t task_id, const std::vector<uint8_t>& hash) override;
  void SearchSuggestionSelected(int32_t task_version, int64_t task_id, const std::vector<uint8_t>& hash) override;
  void OptOutOfSearchSuggestions() override;
};
class  EmbeddedSearchAsyncWaiter {
 public:
  explicit EmbeddedSearchAsyncWaiter(EmbeddedSearch* proxy);
  ~EmbeddedSearchAsyncWaiter();
  void AddCustomLink(
      int32_t page_seq_no, const GURL& url, const std::string& title, bool* out_success);
  void UpdateCustomLink(
      int32_t page_seq_no, const GURL& url, const GURL& new_url, const std::string& new_title, bool* out_success);
  void DeleteCustomLink(
      int32_t page_seq_no, const GURL& url, bool* out_success);
  void HistorySyncCheck(
      int32_t page_seq_no, bool* out_sync_history);
  void ChromeIdentityCheck(
      int32_t page_seq_no, const base::string16& identity, bool* out_identity_match);

 private:
  EmbeddedSearch* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(EmbeddedSearchAsyncWaiter);
};


class  EmbeddedSearchClientInterceptorForTesting : public EmbeddedSearchClient {
  virtual EmbeddedSearchClient* GetForwardingInterface() = 0;
  void SetPageSequenceNumber(int32_t page_seq_no) override;
  void FocusChanged(::OmniboxFocusState new_focus_state, ::OmniboxFocusChangeReason reason) override;
  void MostVisitedChanged(const std::vector<::InstantMostVisitedItem>& items, bool is_custom_links) override;
  void SetInputInProgress(bool input_in_progress) override;
  void ThemeChanged(const ::ThemeBackgroundInfo& value) override;
};
class  EmbeddedSearchClientAsyncWaiter {
 public:
  explicit EmbeddedSearchClientAsyncWaiter(EmbeddedSearchClient* proxy);
  ~EmbeddedSearchClientAsyncWaiter();

 private:
  EmbeddedSearchClient* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(EmbeddedSearchClientAsyncWaiter);
};


class  SearchBouncerInterceptorForTesting : public SearchBouncer {
  virtual SearchBouncer* GetForwardingInterface() = 0;
  void SetNewTabPageURL(const GURL& new_tab_page_url) override;
};
class  SearchBouncerAsyncWaiter {
 public:
  explicit SearchBouncerAsyncWaiter(SearchBouncer* proxy);
  ~SearchBouncerAsyncWaiter();

 private:
  SearchBouncer* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(SearchBouncerAsyncWaiter);
};




}  // namespace mojom
}  // namespace chrome

#endif  // CHROME_COMMON_SEARCH_MOJOM_TEST_UTILS_H_