// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_TEST_UTILS_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_TEST_UTILS_H_

#include "chrome/services/app_service/public/mojom/app_service.mojom.h"


namespace apps {
namespace mojom {


class  AppServiceInterceptorForTesting : public AppService {
  virtual AppService* GetForwardingInterface() = 0;
  void RegisterPublisher(PublisherPtr publisher, ::apps::mojom::AppType app_type) override;
  void RegisterSubscriber(SubscriberPtr subscriber, ConnectOptionsPtr opts) override;
  void LoadIcon(::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, LoadIconCallback callback) override;
  void Launch(::apps::mojom::AppType app_type, const std::string& app_id, int32_t event_flags, ::apps::mojom::LaunchSource launch_source, int64_t display_id) override;
  void SetPermission(::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::PermissionPtr permission) override;
  void Uninstall(::apps::mojom::AppType app_type, const std::string& app_id) override;
  void OpenNativeSettings(::apps::mojom::AppType app_type, const std::string& app_id) override;
};
class  AppServiceAsyncWaiter {
 public:
  explicit AppServiceAsyncWaiter(AppService* proxy);
  ~AppServiceAsyncWaiter();
  void LoadIcon(
      ::apps::mojom::AppType app_type, const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, ::apps::mojom::IconValuePtr* out_icon_value);

 private:
  AppService* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(AppServiceAsyncWaiter);
};


class  PublisherInterceptorForTesting : public Publisher {
  virtual Publisher* GetForwardingInterface() = 0;
  void Connect(SubscriberPtr subscriber, ConnectOptionsPtr opts) override;
  void LoadIcon(const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, LoadIconCallback callback) override;
  void Launch(const std::string& app_id, int32_t event_flags, ::apps::mojom::LaunchSource launch_source, int64_t display_id) override;
  void SetPermission(const std::string& app_id, ::apps::mojom::PermissionPtr permission) override;
  void Uninstall(const std::string& app_id) override;
  void OpenNativeSettings(const std::string& app_id) override;
};
class  PublisherAsyncWaiter {
 public:
  explicit PublisherAsyncWaiter(Publisher* proxy);
  ~PublisherAsyncWaiter();
  void LoadIcon(
      const std::string& app_id, ::apps::mojom::IconKeyPtr icon_key, ::apps::mojom::IconCompression icon_compression, int32_t size_hint_in_dip, bool allow_placeholder_icon, ::apps::mojom::IconValuePtr* out_icon_value);

 private:
  Publisher* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(PublisherAsyncWaiter);
};


class  SubscriberInterceptorForTesting : public Subscriber {
  virtual Subscriber* GetForwardingInterface() = 0;
  void OnApps(std::vector<::apps::mojom::AppPtr> deltas) override;
  void Clone(SubscriberRequest request) override;
};
class  SubscriberAsyncWaiter {
 public:
  explicit SubscriberAsyncWaiter(Subscriber* proxy);
  ~SubscriberAsyncWaiter();

 private:
  Subscriber* const proxy_;

  DISALLOW_COPY_AND_ASSIGN(SubscriberAsyncWaiter);
};




}  // namespace mojom
}  // namespace apps

#endif  // CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_APP_SERVICE_MOJOM_TEST_UTILS_H_