// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_H_
#define CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "chrome/services/app_service/public/mojom/types.mojom-shared.h"
#include "chrome/services/app_service/public/mojom/types.mojom-forward.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include "ui/gfx/image/mojo/image.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace apps {
namespace mojom {






class  Permission {
 public:
  using DataView = PermissionDataView;
  using Data_ = internal::Permission_Data;

  template <typename... Args>
  static PermissionPtr New(Args&&... args) {
    return PermissionPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static PermissionPtr From(const U& u) {
    return mojo::TypeConverter<PermissionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Permission>::Convert(*this);
  }


  Permission();

  Permission(
      uint32_t permission_id,
      PermissionValueType value_type,
      uint32_t value);

  ~Permission();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = PermissionPtr>
  PermissionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Permission>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        Permission::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        Permission::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::Permission_UnserializedMessageContext<
            UserType, Permission::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<Permission::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return Permission::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::Permission_UnserializedMessageContext<
            UserType, Permission::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<Permission::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  uint32_t permission_id;
  
  PermissionValueType value_type;
  
  uint32_t value;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  IconKey {
 public:
  using DataView = IconKeyDataView;
  using Data_ = internal::IconKey_Data;
  
  static constexpr uint64_t kDoesNotChangeOverTime = 0ULL;
  
  static constexpr int32_t kInvalidResourceId = 0;

  template <typename... Args>
  static IconKeyPtr New(Args&&... args) {
    return IconKeyPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static IconKeyPtr From(const U& u) {
    return mojo::TypeConverter<IconKeyPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, IconKey>::Convert(*this);
  }


  IconKey();

  IconKey(
      uint64_t timeline,
      int32_t resource_id,
      uint32_t icon_effects);

  ~IconKey();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = IconKeyPtr>
  IconKeyPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, IconKey>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        IconKey::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        IconKey::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::IconKey_UnserializedMessageContext<
            UserType, IconKey::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<IconKey::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return IconKey::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::IconKey_UnserializedMessageContext<
            UserType, IconKey::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<IconKey::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  uint64_t timeline;
  
  int32_t resource_id;
  
  uint32_t icon_effects;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};








class  App {
 public:
  using DataView = AppDataView;
  using Data_ = internal::App_Data;

  template <typename... Args>
  static AppPtr New(Args&&... args) {
    return AppPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AppPtr From(const U& u) {
    return mojo::TypeConverter<AppPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, App>::Convert(*this);
  }


  App();

  App(
      AppType app_type,
      const std::string& app_id,
      Readiness readiness,
      const base::Optional<std::string>& name,
      const base::Optional<std::string>& short_name,
      const std::vector<std::string>& additional_search_terms,
      IconKeyPtr icon_key,
      base::Optional<base::Time> last_launch_time,
      base::Optional<base::Time> install_time,
      std::vector<PermissionPtr> permissions,
      OptionalBool installed_internally,
      OptionalBool is_platform_app,
      OptionalBool show_in_launcher,
      OptionalBool show_in_search,
      OptionalBool show_in_management);

  ~App();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AppPtr>
  AppPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, App>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        App::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        App::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::App_UnserializedMessageContext<
            UserType, App::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<App::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return App::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::App_UnserializedMessageContext<
            UserType, App::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<App::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  AppType app_type;
  
  std::string app_id;
  
  Readiness readiness;
  
  base::Optional<std::string> name;
  
  base::Optional<std::string> short_name;
  
  std::vector<std::string> additional_search_terms;
  
  IconKeyPtr icon_key;
  
  base::Optional<base::Time> last_launch_time;
  
  base::Optional<base::Time> install_time;
  
  std::vector<PermissionPtr> permissions;
  
  OptionalBool installed_internally;
  
  OptionalBool is_platform_app;
  
  OptionalBool show_in_launcher;
  
  OptionalBool show_in_search;
  
  OptionalBool show_in_management;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(App);
};







class  IconValue {
 public:
  using DataView = IconValueDataView;
  using Data_ = internal::IconValue_Data;

  template <typename... Args>
  static IconValuePtr New(Args&&... args) {
    return IconValuePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static IconValuePtr From(const U& u) {
    return mojo::TypeConverter<IconValuePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, IconValue>::Convert(*this);
  }


  IconValue();

  IconValue(
      IconCompression icon_compression,
      const gfx::ImageSkia& uncompressed,
      const base::Optional<std::vector<uint8_t>>& compressed,
      bool is_placeholder_icon);

  ~IconValue();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = IconValuePtr>
  IconValuePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, IconValue>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        IconValue::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        IconValue::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::IconValue_UnserializedMessageContext<
            UserType, IconValue::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<IconValue::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return IconValue::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::IconValue_UnserializedMessageContext<
            UserType, IconValue::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<IconValue::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  IconCompression icon_compression;
  
  gfx::ImageSkia uncompressed;
  
  base::Optional<std::vector<uint8_t>> compressed;
  
  bool is_placeholder_icon;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};

template <typename StructPtrType>
AppPtr App::Clone() const {
  return New(
      mojo::Clone(app_type),
      mojo::Clone(app_id),
      mojo::Clone(readiness),
      mojo::Clone(name),
      mojo::Clone(short_name),
      mojo::Clone(additional_search_terms),
      mojo::Clone(icon_key),
      mojo::Clone(last_launch_time),
      mojo::Clone(install_time),
      mojo::Clone(permissions),
      mojo::Clone(installed_internally),
      mojo::Clone(is_platform_app),
      mojo::Clone(show_in_launcher),
      mojo::Clone(show_in_search),
      mojo::Clone(show_in_management)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, App>::value>::type*>
bool App::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->app_type, other_struct.app_type))
    return false;
  if (!mojo::Equals(this->app_id, other_struct.app_id))
    return false;
  if (!mojo::Equals(this->readiness, other_struct.readiness))
    return false;
  if (!mojo::Equals(this->name, other_struct.name))
    return false;
  if (!mojo::Equals(this->short_name, other_struct.short_name))
    return false;
  if (!mojo::Equals(this->additional_search_terms, other_struct.additional_search_terms))
    return false;
  if (!mojo::Equals(this->icon_key, other_struct.icon_key))
    return false;
  if (!mojo::Equals(this->last_launch_time, other_struct.last_launch_time))
    return false;
  if (!mojo::Equals(this->install_time, other_struct.install_time))
    return false;
  if (!mojo::Equals(this->permissions, other_struct.permissions))
    return false;
  if (!mojo::Equals(this->installed_internally, other_struct.installed_internally))
    return false;
  if (!mojo::Equals(this->is_platform_app, other_struct.is_platform_app))
    return false;
  if (!mojo::Equals(this->show_in_launcher, other_struct.show_in_launcher))
    return false;
  if (!mojo::Equals(this->show_in_search, other_struct.show_in_search))
    return false;
  if (!mojo::Equals(this->show_in_management, other_struct.show_in_management))
    return false;
  return true;
}
template <typename StructPtrType>
PermissionPtr Permission::Clone() const {
  return New(
      mojo::Clone(permission_id),
      mojo::Clone(value_type),
      mojo::Clone(value)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Permission>::value>::type*>
bool Permission::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->permission_id, other_struct.permission_id))
    return false;
  if (!mojo::Equals(this->value_type, other_struct.value_type))
    return false;
  if (!mojo::Equals(this->value, other_struct.value))
    return false;
  return true;
}
template <typename StructPtrType>
IconKeyPtr IconKey::Clone() const {
  return New(
      mojo::Clone(timeline),
      mojo::Clone(resource_id),
      mojo::Clone(icon_effects)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, IconKey>::value>::type*>
bool IconKey::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->timeline, other_struct.timeline))
    return false;
  if (!mojo::Equals(this->resource_id, other_struct.resource_id))
    return false;
  if (!mojo::Equals(this->icon_effects, other_struct.icon_effects))
    return false;
  return true;
}
template <typename StructPtrType>
IconValuePtr IconValue::Clone() const {
  return New(
      mojo::Clone(icon_compression),
      mojo::Clone(uncompressed),
      mojo::Clone(compressed),
      mojo::Clone(is_placeholder_icon)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, IconValue>::value>::type*>
bool IconValue::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->icon_compression, other_struct.icon_compression))
    return false;
  if (!mojo::Equals(this->uncompressed, other_struct.uncompressed))
    return false;
  if (!mojo::Equals(this->compressed, other_struct.compressed))
    return false;
  if (!mojo::Equals(this->is_placeholder_icon, other_struct.is_placeholder_icon))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace apps

namespace mojo {


template <>
struct  StructTraits<::apps::mojom::App::DataView,
                                         ::apps::mojom::AppPtr> {
  static bool IsNull(const ::apps::mojom::AppPtr& input) { return !input; }
  static void SetToNull(::apps::mojom::AppPtr* output) { output->reset(); }

  static decltype(::apps::mojom::App::app_type) app_type(
      const ::apps::mojom::AppPtr& input) {
    return input->app_type;
  }

  static const decltype(::apps::mojom::App::app_id)& app_id(
      const ::apps::mojom::AppPtr& input) {
    return input->app_id;
  }

  static decltype(::apps::mojom::App::readiness) readiness(
      const ::apps::mojom::AppPtr& input) {
    return input->readiness;
  }

  static const decltype(::apps::mojom::App::name)& name(
      const ::apps::mojom::AppPtr& input) {
    return input->name;
  }

  static const decltype(::apps::mojom::App::short_name)& short_name(
      const ::apps::mojom::AppPtr& input) {
    return input->short_name;
  }

  static const decltype(::apps::mojom::App::additional_search_terms)& additional_search_terms(
      const ::apps::mojom::AppPtr& input) {
    return input->additional_search_terms;
  }

  static const decltype(::apps::mojom::App::icon_key)& icon_key(
      const ::apps::mojom::AppPtr& input) {
    return input->icon_key;
  }

  static const decltype(::apps::mojom::App::last_launch_time)& last_launch_time(
      const ::apps::mojom::AppPtr& input) {
    return input->last_launch_time;
  }

  static const decltype(::apps::mojom::App::install_time)& install_time(
      const ::apps::mojom::AppPtr& input) {
    return input->install_time;
  }

  static const decltype(::apps::mojom::App::permissions)& permissions(
      const ::apps::mojom::AppPtr& input) {
    return input->permissions;
  }

  static decltype(::apps::mojom::App::installed_internally) installed_internally(
      const ::apps::mojom::AppPtr& input) {
    return input->installed_internally;
  }

  static decltype(::apps::mojom::App::is_platform_app) is_platform_app(
      const ::apps::mojom::AppPtr& input) {
    return input->is_platform_app;
  }

  static decltype(::apps::mojom::App::show_in_launcher) show_in_launcher(
      const ::apps::mojom::AppPtr& input) {
    return input->show_in_launcher;
  }

  static decltype(::apps::mojom::App::show_in_search) show_in_search(
      const ::apps::mojom::AppPtr& input) {
    return input->show_in_search;
  }

  static decltype(::apps::mojom::App::show_in_management) show_in_management(
      const ::apps::mojom::AppPtr& input) {
    return input->show_in_management;
  }

  static bool Read(::apps::mojom::App::DataView input, ::apps::mojom::AppPtr* output);
};


template <>
struct  StructTraits<::apps::mojom::Permission::DataView,
                                         ::apps::mojom::PermissionPtr> {
  static bool IsNull(const ::apps::mojom::PermissionPtr& input) { return !input; }
  static void SetToNull(::apps::mojom::PermissionPtr* output) { output->reset(); }

  static decltype(::apps::mojom::Permission::permission_id) permission_id(
      const ::apps::mojom::PermissionPtr& input) {
    return input->permission_id;
  }

  static decltype(::apps::mojom::Permission::value_type) value_type(
      const ::apps::mojom::PermissionPtr& input) {
    return input->value_type;
  }

  static decltype(::apps::mojom::Permission::value) value(
      const ::apps::mojom::PermissionPtr& input) {
    return input->value;
  }

  static bool Read(::apps::mojom::Permission::DataView input, ::apps::mojom::PermissionPtr* output);
};


template <>
struct  StructTraits<::apps::mojom::IconKey::DataView,
                                         ::apps::mojom::IconKeyPtr> {
  static bool IsNull(const ::apps::mojom::IconKeyPtr& input) { return !input; }
  static void SetToNull(::apps::mojom::IconKeyPtr* output) { output->reset(); }

  static decltype(::apps::mojom::IconKey::timeline) timeline(
      const ::apps::mojom::IconKeyPtr& input) {
    return input->timeline;
  }

  static decltype(::apps::mojom::IconKey::resource_id) resource_id(
      const ::apps::mojom::IconKeyPtr& input) {
    return input->resource_id;
  }

  static decltype(::apps::mojom::IconKey::icon_effects) icon_effects(
      const ::apps::mojom::IconKeyPtr& input) {
    return input->icon_effects;
  }

  static bool Read(::apps::mojom::IconKey::DataView input, ::apps::mojom::IconKeyPtr* output);
};


template <>
struct  StructTraits<::apps::mojom::IconValue::DataView,
                                         ::apps::mojom::IconValuePtr> {
  static bool IsNull(const ::apps::mojom::IconValuePtr& input) { return !input; }
  static void SetToNull(::apps::mojom::IconValuePtr* output) { output->reset(); }

  static decltype(::apps::mojom::IconValue::icon_compression) icon_compression(
      const ::apps::mojom::IconValuePtr& input) {
    return input->icon_compression;
  }

  static  decltype(::apps::mojom::IconValue::uncompressed)& uncompressed(
       ::apps::mojom::IconValuePtr& input) {
    return input->uncompressed;
  }

  static const decltype(::apps::mojom::IconValue::compressed)& compressed(
      const ::apps::mojom::IconValuePtr& input) {
    return input->compressed;
  }

  static decltype(::apps::mojom::IconValue::is_placeholder_icon) is_placeholder_icon(
      const ::apps::mojom::IconValuePtr& input) {
    return input->is_placeholder_icon;
  }

  static bool Read(::apps::mojom::IconValue::DataView input, ::apps::mojom::IconValuePtr* output);
};

}  // namespace mojo

#endif  // CHROME_SERVICES_APP_SERVICE_PUBLIC_MOJOM_TYPES_MOJOM_H_