// Copyright 2019 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif


#include "chrome/services/media_gallery_util/public/mojom/media_parser.mojom-test-utils.h"

#include <utility>

#include "base/bind.h"
#include "base/run_loop.h"
#include "media/mojo/interfaces/media_types.mojom.h"
#include "mojo/public/mojom/base/file.mojom.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include "mojo/public/mojom/base/values.mojom.h"


#ifndef CHROME_SERVICES_MEDIA_GALLERY_UTIL_PUBLIC_MOJOM_MEDIA_PARSER_MOJOM_JUMBO_H_
#define CHROME_SERVICES_MEDIA_GALLERY_UTIL_PUBLIC_MOJOM_MEDIA_PARSER_MOJOM_JUMBO_H_
#include "chrome/services/media_gallery_util/public/mojom/media_parser_mojom_traits.h"
#include "gpu/ipc/common/mailbox_holder_struct_traits.h"
#include "gpu/ipc/common/mailbox_struct_traits.h"
#include "gpu/ipc/common/sync_token_struct_traits.h"
#include "media/base/ipc/media_param_traits_macros.h"
#include "media/mojo/interfaces/video_frame_struct_traits.h"
#include "mojo/public/cpp/base/file_mojom_traits.h"
#include "mojo/public/cpp/base/time_mojom_traits.h"
#include "mojo/public/cpp/base/values_mojom_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
#include "ui/gfx/mojo/color_space_mojom_traits.h"
#endif


namespace chrome {
namespace mojom {


void MediaParserInterceptorForTesting::ParseMediaMetadata(const std::string& mime_type, int64_t total_size, bool get_attached_images, MediaDataSourcePtr media_data_source, ParseMediaMetadataCallback callback) {
  GetForwardingInterface()->ParseMediaMetadata(std::move(mime_type), std::move(total_size), std::move(get_attached_images), std::move(media_data_source), std::move(callback));
}
void MediaParserInterceptorForTesting::CheckMediaFile(base::TimeDelta decode_time, base::File file, CheckMediaFileCallback callback) {
  GetForwardingInterface()->CheckMediaFile(std::move(decode_time), std::move(file), std::move(callback));
}
void MediaParserInterceptorForTesting::GetCpuInfo(GetCpuInfoCallback callback) {
  GetForwardingInterface()->GetCpuInfo(std::move(callback));
}
MediaParserAsyncWaiter::MediaParserAsyncWaiter(
    MediaParser* proxy) : proxy_(proxy) {}

MediaParserAsyncWaiter::~MediaParserAsyncWaiter() = default;

void MediaParserAsyncWaiter::ParseMediaMetadata(
    const std::string& mime_type, int64_t total_size, bool get_attached_images, MediaDataSourcePtr media_data_source, bool* out_parse_success, MediaMetadataPtr* out_metadata, std::vector<::metadata::AttachedImage>* out_attached_images) {
  base::RunLoop loop;
  proxy_->ParseMediaMetadata(std::move(mime_type),std::move(total_size),std::move(get_attached_images),std::move(media_data_source),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_parse_success
,
             MediaMetadataPtr* out_metadata
,
             std::vector<::metadata::AttachedImage>* out_attached_images
,
             bool parse_success,
             MediaMetadataPtr metadata,
             const std::vector<::metadata::AttachedImage>& attached_images) {*out_parse_success = std::move(parse_success);*out_metadata = std::move(metadata);*out_attached_images = std::move(attached_images);
            loop->Quit();
          },
          &loop,
          out_parse_success,
          out_metadata,
          out_attached_images));
  loop.Run();
}
void MediaParserAsyncWaiter::CheckMediaFile(
    base::TimeDelta decode_time, base::File file, bool* out_success) {
  base::RunLoop loop;
  proxy_->CheckMediaFile(std::move(decode_time),std::move(file),
      base::BindOnce(
          [](base::RunLoop* loop,
             bool* out_success
,
             bool success) {*out_success = std::move(success);
            loop->Quit();
          },
          &loop,
          out_success));
  loop.Run();
}
void MediaParserAsyncWaiter::GetCpuInfo(
    int64_t* out_libyuv_cpu_flags, int64_t* out_ffmpeg_cpu_flags) {
  base::RunLoop loop;
  proxy_->GetCpuInfo(
      base::BindOnce(
          [](base::RunLoop* loop,
             int64_t* out_libyuv_cpu_flags
,
             int64_t* out_ffmpeg_cpu_flags
,
             int64_t libyuv_cpu_flags,
             int64_t ffmpeg_cpu_flags) {*out_libyuv_cpu_flags = std::move(libyuv_cpu_flags);*out_ffmpeg_cpu_flags = std::move(ffmpeg_cpu_flags);
            loop->Quit();
          },
          &loop,
          out_libyuv_cpu_flags,
          out_ffmpeg_cpu_flags));
  loop.Run();
}



void MediaParserFactoryInterceptorForTesting::CreateMediaParser(int64_t libyuv_cpu_flags, int64_t libavutil_cpu_flags, CreateMediaParserCallback callback) {
  GetForwardingInterface()->CreateMediaParser(std::move(libyuv_cpu_flags), std::move(libavutil_cpu_flags), std::move(callback));
}
MediaParserFactoryAsyncWaiter::MediaParserFactoryAsyncWaiter(
    MediaParserFactory* proxy) : proxy_(proxy) {}

MediaParserFactoryAsyncWaiter::~MediaParserFactoryAsyncWaiter() = default;

void MediaParserFactoryAsyncWaiter::CreateMediaParser(
    int64_t libyuv_cpu_flags, int64_t libavutil_cpu_flags, MediaParserPtr* out_media_parser) {
  base::RunLoop loop;
  proxy_->CreateMediaParser(std::move(libyuv_cpu_flags),std::move(libavutil_cpu_flags),
      base::BindOnce(
          [](base::RunLoop* loop,
             MediaParserPtr* out_media_parser
,
             MediaParserPtr media_parser) {*out_media_parser = std::move(media_parser);
            loop->Quit();
          },
          &loop,
          out_media_parser));
  loop.Run();
}



void MediaDataSourceInterceptorForTesting::Read(int64_t position, int64_t length, ReadCallback callback) {
  GetForwardingInterface()->Read(std::move(position), std::move(length), std::move(callback));
}
MediaDataSourceAsyncWaiter::MediaDataSourceAsyncWaiter(
    MediaDataSource* proxy) : proxy_(proxy) {}

MediaDataSourceAsyncWaiter::~MediaDataSourceAsyncWaiter() = default;

void MediaDataSourceAsyncWaiter::Read(
    int64_t position, int64_t length, std::vector<uint8_t>* out_data) {
  base::RunLoop loop;
  proxy_->Read(std::move(position),std::move(length),
      base::BindOnce(
          [](base::RunLoop* loop,
             std::vector<uint8_t>* out_data
,
             const std::vector<uint8_t>& data) {*out_data = std::move(data);
            loop->Quit();
          },
          &loop,
          out_data));
  loop.Run();
}





}  // namespace mojom
}  // namespace chrome

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif