// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_CHROME_CLEANER_PUBLIC_INTERFACES_CHROME_PROMPT_MOJOM_SHARED_H_
#define COMPONENTS_CHROME_CLEANER_PUBLIC_INTERFACES_CHROME_PROMPT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "components/chrome_cleaner/public/interfaces/chrome_prompt.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace chrome_cleaner {
namespace mojom {
class FilePathDataView;

class RegistryKeyDataView;

class ExtensionIdDataView;



}  // namespace mojom
}  // namespace chrome_cleaner

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::chrome_cleaner::mojom::FilePathDataView> {
  using Data = ::chrome_cleaner::mojom::internal::FilePath_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::chrome_cleaner::mojom::RegistryKeyDataView> {
  using Data = ::chrome_cleaner::mojom::internal::RegistryKey_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::chrome_cleaner::mojom::ExtensionIdDataView> {
  using Data = ::chrome_cleaner::mojom::internal::ExtensionId_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace chrome_cleaner {
namespace mojom {


enum class PromptAcceptance : int32_t {
  
  UNSPECIFIED = 0,
  
  ACCEPTED_WITH_LOGS = 1,
  
  ACCEPTED_WITHOUT_LOGS = 2,
  
  DENIED = 3,
  
  NUM_VALUES,
  kMinValue = 0,
  kMaxValue = 4,
};

 std::ostream& operator<<(std::ostream& os, PromptAcceptance value);
inline bool IsKnownEnumValue(PromptAcceptance value) {
  return internal::PromptAcceptance_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class ChromePromptInterfaceBase {};

using ChromePromptPtrDataView =
    mojo::InterfacePtrDataView<ChromePromptInterfaceBase>;
using ChromePromptRequestDataView =
    mojo::InterfaceRequestDataView<ChromePromptInterfaceBase>;
using ChromePromptAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ChromePromptInterfaceBase>;
using ChromePromptAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ChromePromptInterfaceBase>;
class FilePathDataView {
 public:
  FilePathDataView() {}

  FilePathDataView(
      internal::FilePath_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetValueDataView(
      mojo::ArrayDataView<uint16_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValue(UserType* output) {
    auto* pointer = data_->value.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint16_t>>(
        pointer, output, context_);
  }
 private:
  internal::FilePath_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class RegistryKeyDataView {
 public:
  RegistryKeyDataView() {}

  RegistryKeyDataView(
      internal::RegistryKey_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetValueDataView(
      mojo::ArrayDataView<uint16_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValue(UserType* output) {
    auto* pointer = data_->value.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint16_t>>(
        pointer, output, context_);
  }
 private:
  internal::RegistryKey_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ExtensionIdDataView {
 public:
  ExtensionIdDataView() {}

  ExtensionIdDataView(
      internal::ExtensionId_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetValueDataView(
      mojo::ArrayDataView<uint16_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValue(UserType* output) {
    auto* pointer = data_->value.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint16_t>>(
        pointer, output, context_);
  }
 private:
  internal::ExtensionId_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace chrome_cleaner

namespace std {

template <>
struct hash<::chrome_cleaner::mojom::PromptAcceptance>
    : public mojo::internal::EnumHashImpl<::chrome_cleaner::mojom::PromptAcceptance> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::chrome_cleaner::mojom::PromptAcceptance, ::chrome_cleaner::mojom::PromptAcceptance> {
  static ::chrome_cleaner::mojom::PromptAcceptance ToMojom(::chrome_cleaner::mojom::PromptAcceptance input) { return input; }
  static bool FromMojom(::chrome_cleaner::mojom::PromptAcceptance input, ::chrome_cleaner::mojom::PromptAcceptance* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome_cleaner::mojom::PromptAcceptance, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::chrome_cleaner::mojom::PromptAcceptance, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::chrome_cleaner::mojom::PromptAcceptance>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome_cleaner::mojom::FilePathDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::chrome_cleaner::mojom::FilePathDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::chrome_cleaner::mojom::internal::FilePath_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::value(input)) in_value = Traits::value(input);
    typename decltype((*output)->value)::BaseType::BufferWriter
        value_writer;
    const mojo::internal::ContainerValidateParams value_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
        in_value, buffer, &value_writer, &value_validate_params,
        context);
    (*output)->value.Set(
        value_writer.is_null() ? nullptr : value_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->value.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null value in FilePath struct");
  }

  static bool Deserialize(::chrome_cleaner::mojom::internal::FilePath_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::chrome_cleaner::mojom::FilePathDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome_cleaner::mojom::RegistryKeyDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::chrome_cleaner::mojom::RegistryKeyDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::chrome_cleaner::mojom::internal::RegistryKey_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::value(input)) in_value = Traits::value(input);
    typename decltype((*output)->value)::BaseType::BufferWriter
        value_writer;
    const mojo::internal::ContainerValidateParams value_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
        in_value, buffer, &value_writer, &value_validate_params,
        context);
    (*output)->value.Set(
        value_writer.is_null() ? nullptr : value_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->value.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null value in RegistryKey struct");
  }

  static bool Deserialize(::chrome_cleaner::mojom::internal::RegistryKey_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::chrome_cleaner::mojom::RegistryKeyDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::chrome_cleaner::mojom::ExtensionIdDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::chrome_cleaner::mojom::ExtensionIdDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::chrome_cleaner::mojom::internal::ExtensionId_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::value(input)) in_value = Traits::value(input);
    typename decltype((*output)->value)::BaseType::BufferWriter
        value_writer;
    const mojo::internal::ContainerValidateParams value_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint16_t>>(
        in_value, buffer, &value_writer, &value_validate_params,
        context);
    (*output)->value.Set(
        value_writer.is_null() ? nullptr : value_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->value.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null value in ExtensionId struct");
  }

  static bool Deserialize(::chrome_cleaner::mojom::internal::ExtensionId_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::chrome_cleaner::mojom::ExtensionIdDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace chrome_cleaner {
namespace mojom {

inline void FilePathDataView::GetValueDataView(
    mojo::ArrayDataView<uint16_t>* output) {
  auto pointer = data_->value.Get();
  *output = mojo::ArrayDataView<uint16_t>(pointer, context_);
}


inline void RegistryKeyDataView::GetValueDataView(
    mojo::ArrayDataView<uint16_t>* output) {
  auto pointer = data_->value.Get();
  *output = mojo::ArrayDataView<uint16_t>(pointer, context_);
}


inline void ExtensionIdDataView::GetValueDataView(
    mojo::ArrayDataView<uint16_t>* output) {
  auto pointer = data_->value.Get();
  *output = mojo::ArrayDataView<uint16_t>(pointer, context_);
}



}  // namespace mojom
}  // namespace chrome_cleaner

#endif  // COMPONENTS_CHROME_CLEANER_PUBLIC_INTERFACES_CHROME_PROMPT_MOJOM_SHARED_H_