// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_METRICS_PUBLIC_INTERFACES_CALL_STACK_PROFILE_COLLECTOR_MOJOM_H_
#define COMPONENTS_METRICS_PUBLIC_INTERFACES_CALL_STACK_PROFILE_COLLECTOR_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "components/metrics/public/interfaces/call_stack_profile_collector.mojom-shared.h"
#include "components/metrics/public/interfaces/call_stack_profile_collector.mojom-forward.h"
#include "mojo/public/mojom/base/time.mojom.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace metrics {
namespace mojom {

class CallStackProfileCollectorProxy;

template <typename ImplRefTraits>
class CallStackProfileCollectorStub;

class CallStackProfileCollectorRequestValidator;


class  CallStackProfileCollector
    : public CallStackProfileCollectorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CallStackProfileCollectorProxy;

  template <typename ImplRefTraits>
  using Stub_ = CallStackProfileCollectorStub<ImplRefTraits>;

  using RequestValidator_ = CallStackProfileCollectorRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCollectMinVersion = 0,
  };
  virtual ~CallStackProfileCollector() {}

  
  virtual void Collect(base::TimeTicks start_timestamp, SampledProfilePtr profile) = 0;
};

class  CallStackProfileCollectorProxy
    : public CallStackProfileCollector {
 public:
  using InterfaceType = CallStackProfileCollector;

  explicit CallStackProfileCollectorProxy(mojo::MessageReceiverWithResponder* receiver);
  void Collect(base::TimeTicks start_timestamp, SampledProfilePtr profile) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  CallStackProfileCollectorStubDispatch {
 public:
  static bool Accept(CallStackProfileCollector* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CallStackProfileCollector* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CallStackProfileCollector>>
class CallStackProfileCollectorStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CallStackProfileCollectorStub() {}
  ~CallStackProfileCollectorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CallStackProfileCollectorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CallStackProfileCollectorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CallStackProfileCollectorRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class  SampledProfile {
 public:
  using DataView = SampledProfileDataView;
  using Data_ = internal::SampledProfile_Data;

  template <typename... Args>
  static SampledProfilePtr New(Args&&... args) {
    return SampledProfilePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static SampledProfilePtr From(const U& u) {
    return mojo::TypeConverter<SampledProfilePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SampledProfile>::Convert(*this);
  }


  SampledProfile();

  explicit SampledProfile(
      const std::string& contents);

  ~SampledProfile();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SampledProfilePtr>
  SampledProfilePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SampledProfile>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        SampledProfile::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        SampledProfile::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::SampledProfile_UnserializedMessageContext<
            UserType, SampledProfile::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<SampledProfile::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return SampledProfile::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::SampledProfile_UnserializedMessageContext<
            UserType, SampledProfile::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<SampledProfile::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string contents;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
SampledProfilePtr SampledProfile::Clone() const {
  return New(
      mojo::Clone(contents)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SampledProfile>::value>::type*>
bool SampledProfile::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->contents, other_struct.contents))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace metrics

namespace mojo {


template <>
struct  StructTraits<::metrics::mojom::SampledProfile::DataView,
                                         ::metrics::mojom::SampledProfilePtr> {
  static bool IsNull(const ::metrics::mojom::SampledProfilePtr& input) { return !input; }
  static void SetToNull(::metrics::mojom::SampledProfilePtr* output) { output->reset(); }

  static const decltype(::metrics::mojom::SampledProfile::contents)& contents(
      const ::metrics::mojom::SampledProfilePtr& input) {
    return input->contents;
  }

  static bool Read(::metrics::mojom::SampledProfile::DataView input, ::metrics::mojom::SampledProfilePtr* output);
};

}  // namespace mojo

#endif  // COMPONENTS_METRICS_PUBLIC_INTERFACES_CALL_STACK_PROFILE_COLLECTOR_MOJOM_H_