// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_SHARED_H_
#define COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "components/mirroring/mojom/cast_message_channel.mojom-shared-internal.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace mirroring {
namespace mojom {
class CastMessageDataView;



}  // namespace mojom
}  // namespace mirroring

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::mirroring::mojom::CastMessageDataView> {
  using Data = ::mirroring::mojom::internal::CastMessage_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace mirroring {
namespace mojom {
// Interface base classes. They are used for type safety check.
class CastMessageChannelInterfaceBase {};

using CastMessageChannelPtrDataView =
    mojo::InterfacePtrDataView<CastMessageChannelInterfaceBase>;
using CastMessageChannelRequestDataView =
    mojo::InterfaceRequestDataView<CastMessageChannelInterfaceBase>;
using CastMessageChannelAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CastMessageChannelInterfaceBase>;
using CastMessageChannelAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CastMessageChannelInterfaceBase>;
class CastMessageDataView {
 public:
  CastMessageDataView() {}

  CastMessageDataView(
      internal::CastMessage_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMessageNamespaceDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMessageNamespace(UserType* output) {
    auto* pointer = data_->message_namespace.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetJsonFormatDataDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadJsonFormatData(UserType* output) {
    auto* pointer = data_->json_format_data.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::CastMessage_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace mirroring

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mirroring::mojom::CastMessageDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mirroring::mojom::CastMessageDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mirroring::mojom::internal::CastMessage_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::message_namespace(input)) in_message_namespace = Traits::message_namespace(input);
    typename decltype((*output)->message_namespace)::BaseType::BufferWriter
        message_namespace_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_message_namespace, buffer, &message_namespace_writer, context);
    (*output)->message_namespace.Set(
        message_namespace_writer.is_null() ? nullptr : message_namespace_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->message_namespace.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null message_namespace in CastMessage struct");
    decltype(Traits::json_format_data(input)) in_json_format_data = Traits::json_format_data(input);
    typename decltype((*output)->json_format_data)::BaseType::BufferWriter
        json_format_data_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_json_format_data, buffer, &json_format_data_writer, context);
    (*output)->json_format_data.Set(
        json_format_data_writer.is_null() ? nullptr : json_format_data_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->json_format_data.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null json_format_data in CastMessage struct");
  }

  static bool Deserialize(::mirroring::mojom::internal::CastMessage_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mirroring::mojom::CastMessageDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace mirroring {
namespace mojom {

inline void CastMessageDataView::GetMessageNamespaceDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->message_namespace.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void CastMessageDataView::GetJsonFormatDataDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->json_format_data.Get();
  *output = mojo::StringDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace mirroring

#endif  // COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_SHARED_H_