// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "components/mirroring/mojom/cast_message_channel.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "base/run_loop.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"

#include "components/mirroring/mojom/cast_message_channel.mojom-params-data.h"
#include "components/mirroring/mojom/cast_message_channel.mojom-shared-message-ids.h"

#include "components/mirroring/mojom/cast_message_channel.mojom-import-headers.h"


#ifndef COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_JUMBO_H_
#define COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_JUMBO_H_
#endif
namespace mirroring {
namespace mojom {
const char kRemotingNamespace[] = "urn:x-cast:com.google.cast.remoting";
const char kWebRtcNamespace[] = "urn:x-cast:com.google.cast.webrtc";
CastMessage::CastMessage()
    : message_namespace(),
      json_format_data() {}

CastMessage::CastMessage(
    const std::string& message_namespace_in,
    const std::string& json_format_data_in)
    : message_namespace(std::move(message_namespace_in)),
      json_format_data(std::move(json_format_data_in)) {}

CastMessage::~CastMessage() = default;
size_t CastMessage::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->message_namespace);
  seed = mojo::internal::Hash(seed, this->json_format_data);
  return seed;
}

bool CastMessage::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char CastMessageChannel::Name_[] = "mirroring.mojom.CastMessageChannel";

CastMessageChannelProxy::CastMessageChannelProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void CastMessageChannelProxy::Send(
    CastMessagePtr in_message) {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT0("mojom", "<class 'jinja2::utils::Namespace'>::CastMessageChannel::Send");
#endif
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::Message message(
      internal::kCastMessageChannel_Send_Name, kFlags, 0, 0, nullptr);
  auto* buffer = message.payload_buffer();
  ::mirroring::mojom::internal::CastMessageChannel_Send_Params_Data::BufferWriter
      params;
  mojo::internal::SerializationContext serialization_context;
  params.Allocate(buffer);
  typename decltype(params->message)::BaseType::BufferWriter
      message_writer;
  mojo::internal::Serialize<::mirroring::mojom::CastMessageDataView>(
      in_message, buffer, &message_writer, &serialization_context);
  params->message.Set(
      message_writer.is_null() ? nullptr : message_writer.data());
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->message.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null message in CastMessageChannel.Send request");
  message.AttachHandlesFromSerializationContext(
      &serialization_context);

#if defined(ENABLE_IPC_FUZZER)
  message.set_interface_name(CastMessageChannel::Name_);
  message.set_method_name("Send");
#endif
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool CastMessageChannelStubDispatch::Accept(
    CastMessageChannel* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kCastMessageChannel_Send_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)<class 'jinja2::utils::Namespace'>::CastMessageChannel::Send",
               "message", message->name());
#endif
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::CastMessageChannel_Send_Params_Data* params =
          reinterpret_cast<internal::CastMessageChannel_Send_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      CastMessagePtr p_message{};
      CastMessageChannel_Send_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadMessage(&p_message))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "CastMessageChannel::Send deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      DCHECK(impl);
      impl->Send(
std::move(p_message));
      return true;
    }
  }
  return false;
}

// static
bool CastMessageChannelStubDispatch::AcceptWithResponder(
    CastMessageChannel* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kCastMessageChannel_Send_Name: {
#if BUILDFLAG(MOJO_TRACE_ENABLED)
  TRACE_EVENT1("mojom", "(Impl)<class 'jinja2::utils::Namespace'>::CastMessageChannel::Send",
               "message", message->name());
#endif
      break;
    }
  }
  return false;
}

bool CastMessageChannelRequestValidator::Accept(mojo::Message* message) {
  if (!message->is_serialized() ||
      mojo::internal::ControlMessageHandler::IsControlMessage(message)) {
    return true;
  }

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "CastMessageChannel RequestValidator");

  switch (message->header()->name) {
    case internal::kCastMessageChannel_Send_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::CastMessageChannel_Send_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace mirroring

namespace mojo {


// static
bool StructTraits<::mirroring::mojom::CastMessage::DataView, ::mirroring::mojom::CastMessagePtr>::Read(
    ::mirroring::mojom::CastMessage::DataView input,
    ::mirroring::mojom::CastMessagePtr* output) {
  bool success = true;
  ::mirroring::mojom::CastMessagePtr result(::mirroring::mojom::CastMessage::New());
  
      if (!input.ReadMessageNamespace(&result->message_namespace))
        success = false;
      if (!input.ReadJsonFormatData(&result->json_format_data))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif