// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_H_
#define COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "components/mirroring/mojom/cast_message_channel.mojom-shared.h"
#include "components/mirroring/mojom/cast_message_channel.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace mirroring {
namespace mojom {

class CastMessageChannelProxy;

template <typename ImplRefTraits>
class CastMessageChannelStub;

class CastMessageChannelRequestValidator;


class  CastMessageChannel
    : public CastMessageChannelInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CastMessageChannelProxy;

  template <typename ImplRefTraits>
  using Stub_ = CastMessageChannelStub<ImplRefTraits>;

  using RequestValidator_ = CastMessageChannelRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSendMinVersion = 0,
  };
  virtual ~CastMessageChannel() {}

  
  virtual void Send(CastMessagePtr message) = 0;
};

class  CastMessageChannelProxy
    : public CastMessageChannel {
 public:
  using InterfaceType = CastMessageChannel;

  explicit CastMessageChannelProxy(mojo::MessageReceiverWithResponder* receiver);
  void Send(CastMessagePtr message) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  CastMessageChannelStubDispatch {
 public:
  static bool Accept(CastMessageChannel* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CastMessageChannel* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CastMessageChannel>>
class CastMessageChannelStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CastMessageChannelStub() {}
  ~CastMessageChannelStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CastMessageChannelStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CastMessageChannelStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CastMessageChannelRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class  CastMessage {
 public:
  using DataView = CastMessageDataView;
  using Data_ = internal::CastMessage_Data;

  template <typename... Args>
  static CastMessagePtr New(Args&&... args) {
    return CastMessagePtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static CastMessagePtr From(const U& u) {
    return mojo::TypeConverter<CastMessagePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CastMessage>::Convert(*this);
  }


  CastMessage();

  CastMessage(
      const std::string& message_namespace,
      const std::string& json_format_data);

  ~CastMessage();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CastMessagePtr>
  CastMessagePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CastMessage>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        CastMessage::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        CastMessage::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::CastMessage_UnserializedMessageContext<
            UserType, CastMessage::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<CastMessage::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return CastMessage::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::CastMessage_UnserializedMessageContext<
            UserType, CastMessage::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<CastMessage::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  std::string message_namespace;
  
  std::string json_format_data;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
CastMessagePtr CastMessage::Clone() const {
  return New(
      mojo::Clone(message_namespace),
      mojo::Clone(json_format_data)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CastMessage>::value>::type*>
bool CastMessage::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->message_namespace, other_struct.message_namespace))
    return false;
  if (!mojo::Equals(this->json_format_data, other_struct.json_format_data))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace mirroring

namespace mojo {


template <>
struct  StructTraits<::mirroring::mojom::CastMessage::DataView,
                                         ::mirroring::mojom::CastMessagePtr> {
  static bool IsNull(const ::mirroring::mojom::CastMessagePtr& input) { return !input; }
  static void SetToNull(::mirroring::mojom::CastMessagePtr* output) { output->reset(); }

  static const decltype(::mirroring::mojom::CastMessage::message_namespace)& message_namespace(
      const ::mirroring::mojom::CastMessagePtr& input) {
    return input->message_namespace;
  }

  static const decltype(::mirroring::mojom::CastMessage::json_format_data)& json_format_data(
      const ::mirroring::mojom::CastMessagePtr& input) {
    return input->json_format_data;
  }

  static bool Read(::mirroring::mojom::CastMessage::DataView input, ::mirroring::mojom::CastMessagePtr* output);
};

}  // namespace mojo

#endif  // COMPONENTS_MIRRORING_MOJOM_CAST_MESSAGE_CHANNEL_MOJOM_H_