// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "services/identity/public/mojom/account_info.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "base/stl_util.h"  // for base::size()
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"

#include "services/identity/public/mojom/account_info.mojom-params-data.h"
namespace identity {
namespace mojom {

namespace internal {


// static
bool AccountInfo_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const AccountInfo_Data* object = static_cast<const AccountInfo_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 80 }};

  if (object->header_.version <=
          kVersionSizes[base::size(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = base::size(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[base::size(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->account_id, 1, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams account_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->account_id, validation_context,
                                         &account_id_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->gaia, 2, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams gaia_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->gaia, validation_context,
                                         &gaia_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->email, 3, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams email_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->email, validation_context,
                                         &email_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->full_name, 4, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams full_name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->full_name, validation_context,
                                         &full_name_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->given_name, 5, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams given_name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->given_name, validation_context,
                                         &given_name_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->hosted_domain, 6, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams hosted_domain_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->hosted_domain, validation_context,
                                         &hosted_domain_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->locale, 7, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams locale_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->locale, validation_context,
                                         &locale_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->picture_url, 8, validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams picture_url_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->picture_url, validation_context,
                                         &picture_url_validate_params)) {
    return false;
  }

  return true;
}

AccountInfo_Data::AccountInfo_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace identity

#if defined(_MSC_VER)
#pragma warning(pop)
#endif