// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_IDENTITY_PUBLIC_MOJOM_IDENTITY_ACCESSOR_MOJOM_H_
#define SERVICES_IDENTITY_PUBLIC_MOJOM_IDENTITY_ACCESSOR_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/identity/public/mojom/identity_accessor.mojom-shared.h"
#include "services/identity/public/mojom/identity_accessor.mojom-forward.h"
#include "mojo/public/mojom/base/time.mojom-forward.h"
#include "services/identity/public/mojom/account.mojom-forward.h"
#include "services/identity/public/mojom/account_info.mojom-forward.h"
#include "services/identity/public/mojom/account_state.mojom-forward.h"
#include "services/identity/public/mojom/google_service_auth_error.mojom-forward.h"
#include "services/identity/public/mojom/scope_set.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"




namespace identity {
namespace mojom {

class IdentityAccessorProxy;

template <typename ImplRefTraits>
class IdentityAccessorStub;

class IdentityAccessorRequestValidator;
class IdentityAccessorResponseValidator;


class  IdentityAccessor
    : public IdentityAccessorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = IdentityAccessorProxy;

  template <typename ImplRefTraits>
  using Stub_ = IdentityAccessorStub<ImplRefTraits>;

  using RequestValidator_ = IdentityAccessorRequestValidator;
  using ResponseValidator_ = IdentityAccessorResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGetPrimaryAccountInfoMinVersion = 0,
    kGetPrimaryAccountWhenAvailableMinVersion = 0,
    kGetAccessTokenMinVersion = 0,
  };
  virtual ~IdentityAccessor() {}


  using GetPrimaryAccountInfoCallback = base::OnceCallback<void(const base::Optional<::AccountInfo>&, const identity::AccountState&)>;
  
  virtual void GetPrimaryAccountInfo(GetPrimaryAccountInfoCallback callback) = 0;


  using GetPrimaryAccountWhenAvailableCallback = base::OnceCallback<void(const ::AccountInfo&, const identity::AccountState&)>;
  
  virtual void GetPrimaryAccountWhenAvailable(GetPrimaryAccountWhenAvailableCallback callback) = 0;


  using GetAccessTokenCallback = base::OnceCallback<void(const base::Optional<std::string>&, base::Time, const ::GoogleServiceAuthError&)>;
  
  virtual void GetAccessToken(const std::string& account_id, const ::identity::ScopeSet& scopes, const std::string& consumer_id, GetAccessTokenCallback callback) = 0;
};

class  IdentityAccessorProxy
    : public IdentityAccessor {
 public:
  using InterfaceType = IdentityAccessor;

  explicit IdentityAccessorProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetPrimaryAccountInfo(GetPrimaryAccountInfoCallback callback) final;
  void GetPrimaryAccountWhenAvailable(GetPrimaryAccountWhenAvailableCallback callback) final;
  void GetAccessToken(const std::string& account_id, const ::identity::ScopeSet& scopes, const std::string& consumer_id, GetAccessTokenCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  IdentityAccessorStubDispatch {
 public:
  static bool Accept(IdentityAccessor* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      IdentityAccessor* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<IdentityAccessor>>
class IdentityAccessorStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  IdentityAccessorStub() {}
  ~IdentityAccessorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return IdentityAccessorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return IdentityAccessorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  IdentityAccessorRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class  IdentityAccessorResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace identity

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_IDENTITY_PUBLIC_MOJOM_IDENTITY_ACCESSOR_MOJOM_H_