// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_COORDINATION_UNIT_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_COORDINATION_UNIT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/mojom/coordination_unit.mojom-shared.h"
#include "services/resource_coordinator/public/mojom/coordination_unit.mojom-forward.h"
#include "mojo/public/mojom/base/process_id.mojom-forward.h"
#include "mojo/public/mojom/base/time.mojom-forward.h"
#include "services/resource_coordinator/public/mojom/lifecycle.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_id.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_types.h"
#include "services/resource_coordinator/public/cpp/page_navigation_identity.h"
#include "base/component_export.h"




namespace resource_coordinator {
namespace mojom {

class FrameCoordinationUnitProxy;

template <typename ImplRefTraits>
class FrameCoordinationUnitStub;

class FrameCoordinationUnitRequestValidator;


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) FrameCoordinationUnit
    : public FrameCoordinationUnitInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameCoordinationUnitProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameCoordinationUnitStub<ImplRefTraits>;

  using RequestValidator_ = FrameCoordinationUnitRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetNetworkAlmostIdleMinVersion = 0,
    kSetLifecycleStateMinVersion = 0,
    kSetHasNonEmptyBeforeUnloadMinVersion = 0,
    kSetInterventionPolicyMinVersion = 0,
    kOnNonPersistentNotificationCreatedMinVersion = 0,
  };
  virtual ~FrameCoordinationUnit() {}

  
  virtual void SetNetworkAlmostIdle(bool idle) = 0;

  
  virtual void SetLifecycleState(::resource_coordinator::mojom::LifecycleState state) = 0;

  
  virtual void SetHasNonEmptyBeforeUnload(bool has_nonempty_beforeunload) = 0;

  
  virtual void SetInterventionPolicy(PolicyControlledIntervention intervention, InterventionPolicy policy) = 0;

  
  virtual void OnNonPersistentNotificationCreated() = 0;
};

class ProcessCoordinationUnitProxy;

template <typename ImplRefTraits>
class ProcessCoordinationUnitStub;

class ProcessCoordinationUnitRequestValidator;


class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) ProcessCoordinationUnit
    : public ProcessCoordinationUnitInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ProcessCoordinationUnitProxy;

  template <typename ImplRefTraits>
  using Stub_ = ProcessCoordinationUnitStub<ImplRefTraits>;

  using RequestValidator_ = ProcessCoordinationUnitRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetExpectedTaskQueueingDurationMinVersion = 0,
    kSetMainThreadTaskLoadIsLowMinVersion = 0,
  };
  virtual ~ProcessCoordinationUnit() {}

  
  virtual void SetExpectedTaskQueueingDuration(base::TimeDelta duration) = 0;

  
  virtual void SetMainThreadTaskLoadIsLow(bool main_thread_task_load_is_low) = 0;
};

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) FrameCoordinationUnitProxy
    : public FrameCoordinationUnit {
 public:
  using InterfaceType = FrameCoordinationUnit;

  explicit FrameCoordinationUnitProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetNetworkAlmostIdle(bool idle) final;
  void SetLifecycleState(::resource_coordinator::mojom::LifecycleState state) final;
  void SetHasNonEmptyBeforeUnload(bool has_nonempty_beforeunload) final;
  void SetInterventionPolicy(PolicyControlledIntervention intervention, InterventionPolicy policy) final;
  void OnNonPersistentNotificationCreated() final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) ProcessCoordinationUnitProxy
    : public ProcessCoordinationUnit {
 public:
  using InterfaceType = ProcessCoordinationUnit;

  explicit ProcessCoordinationUnitProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetExpectedTaskQueueingDuration(base::TimeDelta duration) final;
  void SetMainThreadTaskLoadIsLow(bool main_thread_task_load_is_low) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) FrameCoordinationUnitStubDispatch {
 public:
  static bool Accept(FrameCoordinationUnit* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameCoordinationUnit* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameCoordinationUnit>>
class FrameCoordinationUnitStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameCoordinationUnitStub() {}
  ~FrameCoordinationUnitStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameCoordinationUnitStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameCoordinationUnitStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) ProcessCoordinationUnitStubDispatch {
 public:
  static bool Accept(ProcessCoordinationUnit* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ProcessCoordinationUnit* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ProcessCoordinationUnit>>
class ProcessCoordinationUnitStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ProcessCoordinationUnitStub() {}
  ~ProcessCoordinationUnitStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProcessCoordinationUnitStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ProcessCoordinationUnitStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) FrameCoordinationUnitRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) ProcessCoordinationUnitRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





class COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) CoordinationUnitID {
 public:
  using DataView = CoordinationUnitIDDataView;
  using Data_ = internal::CoordinationUnitID_Data;

  template <typename... Args>
  static CoordinationUnitIDPtr New(Args&&... args) {
    return CoordinationUnitIDPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static CoordinationUnitIDPtr From(const U& u) {
    return mojo::TypeConverter<CoordinationUnitIDPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CoordinationUnitID>::Convert(*this);
  }


  CoordinationUnitID();

  CoordinationUnitID(
      ::resource_coordinator::CoordinationUnitType type,
      int64_t id);

  ~CoordinationUnitID();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CoordinationUnitIDPtr>
  CoordinationUnitIDPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CoordinationUnitID>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        CoordinationUnitID::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        CoordinationUnitID::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::CoordinationUnitID_UnserializedMessageContext<
            UserType, CoordinationUnitID::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<CoordinationUnitID::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return CoordinationUnitID::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::CoordinationUnitID_UnserializedMessageContext<
            UserType, CoordinationUnitID::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<CoordinationUnitID::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  ::resource_coordinator::CoordinationUnitType type;
  
  int64_t id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
CoordinationUnitIDPtr CoordinationUnitID::Clone() const {
  return New(
      mojo::Clone(type),
      mojo::Clone(id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CoordinationUnitID>::value>::type*>
bool CoordinationUnitID::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->type, other_struct.type))
    return false;
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


template <>
struct COMPONENT_EXPORT(RESOURCE_COORDINATOR_PUBLIC_MOJOM) StructTraits<::resource_coordinator::mojom::CoordinationUnitID::DataView,
                                         ::resource_coordinator::mojom::CoordinationUnitIDPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::CoordinationUnitIDPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::CoordinationUnitID::type) type(
      const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) {
    return input->type;
  }

  static decltype(::resource_coordinator::mojom::CoordinationUnitID::id) id(
      const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) {
    return input->id;
  }

  static bool Read(::resource_coordinator::mojom::CoordinationUnitID::DataView input, ::resource_coordinator::mojom::CoordinationUnitIDPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_COORDINATION_UNIT_MOJOM_H_