// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_SHARED_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "base/containers/flat_map.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/resource_coordinator/public/mojom/page_signal.mojom-shared-internal.h"
#include "services/resource_coordinator/public/mojom/coordination_unit.mojom-shared.h"
#include "services/resource_coordinator/public/mojom/lifecycle.mojom-shared.h"
#include "mojo/public/mojom/base/time.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/interface_serialization.h"


#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "base/component_export.h"




namespace resource_coordinator {
namespace mojom {
class PageNavigationIdentityDataView;



}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::PageNavigationIdentityDataView> {
  using Data = ::resource_coordinator::mojom::internal::PageNavigationIdentity_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace resource_coordinator {
namespace mojom {
// Interface base classes. They are used for type safety check.
class PageSignalReceiverInterfaceBase {};

using PageSignalReceiverPtrDataView =
    mojo::InterfacePtrDataView<PageSignalReceiverInterfaceBase>;
using PageSignalReceiverRequestDataView =
    mojo::InterfaceRequestDataView<PageSignalReceiverInterfaceBase>;
using PageSignalReceiverAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageSignalReceiverInterfaceBase>;
using PageSignalReceiverAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageSignalReceiverInterfaceBase>;
class PageSignalGeneratorInterfaceBase {};

using PageSignalGeneratorPtrDataView =
    mojo::InterfacePtrDataView<PageSignalGeneratorInterfaceBase>;
using PageSignalGeneratorRequestDataView =
    mojo::InterfaceRequestDataView<PageSignalGeneratorInterfaceBase>;
using PageSignalGeneratorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PageSignalGeneratorInterfaceBase>;
using PageSignalGeneratorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PageSignalGeneratorInterfaceBase>;
class PageNavigationIdentityDataView {
 public:
  PageNavigationIdentityDataView() {}

  PageNavigationIdentityDataView(
      internal::PageNavigationIdentity_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPageCuIdDataView(
      ::resource_coordinator::mojom::CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPageCuId(UserType* output) {
    auto* pointer = data_->page_cu_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
  int64_t navigation_id() const {
    return data_->navigation_id;
  }
  inline void GetUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PageNavigationIdentity_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace resource_coordinator

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::PageNavigationIdentityDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::PageNavigationIdentityDataView, UserType>;

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::PageNavigationIdentity_Data::BufferWriter* output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return;
    (*output).Allocate(buffer);
    decltype(Traits::page_cu_id(input)) in_page_cu_id = Traits::page_cu_id(input);
    typename decltype((*output)->page_cu_id)::BaseType::BufferWriter
        page_cu_id_writer;
    mojo::internal::Serialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        in_page_cu_id, buffer, &page_cu_id_writer, context);
    (*output)->page_cu_id.Set(
        page_cu_id_writer.is_null() ? nullptr : page_cu_id_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->page_cu_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null page_cu_id in PageNavigationIdentity struct");
    (*output)->navigation_id = Traits::navigation_id(input);
    decltype(Traits::url(input)) in_url = Traits::url(input);
    typename decltype((*output)->url)::BaseType::BufferWriter
        url_writer;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_url, buffer, &url_writer, context);
    (*output)->url.Set(
        url_writer.is_null() ? nullptr : url_writer.data());
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        (*output)->url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null url in PageNavigationIdentity struct");
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::PageNavigationIdentity_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::PageNavigationIdentityDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace resource_coordinator {
namespace mojom {

inline void PageNavigationIdentityDataView::GetPageCuIdDataView(
    ::resource_coordinator::mojom::CoordinationUnitIDDataView* output) {
  auto pointer = data_->page_cu_id.Get();
  *output = ::resource_coordinator::mojom::CoordinationUnitIDDataView(pointer, context_);
}
inline void PageNavigationIdentityDataView::GetUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->url.Get();
  *output = mojo::StringDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace resource_coordinator

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_MOJOM_PAGE_SIGNAL_MOJOM_SHARED_H_