// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_WRITER_MOJOM_BLINK_H_
#define THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_WRITER_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/blink/public/mojom/filesystem/file_writer.mojom-shared.h"
#include "third_party/blink/public/mojom/filesystem/file_writer.mojom-blink-forward.h"
#include "mojo/public/mojom/base/file_error.mojom-blink-forward.h"
#include "third_party/blink/public/mojom/blob/blob.mojom-blink-forward.h"

#include "mojo/public/cpp/bindings/lib/wtf_clone_equals_util.h"
#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/text/wtf_string.h"

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "third_party/blink/renderer/platform/platform_export.h"




namespace blink {
namespace mojom {
namespace blink {

class FileWriterProxy;

template <typename ImplRefTraits>
class FileWriterStub;

class FileWriterRequestValidator;
class FileWriterResponseValidator;


class PLATFORM_EXPORT FileWriter
    : public FileWriterInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FileWriterProxy;

  template <typename ImplRefTraits>
  using Stub_ = FileWriterStub<ImplRefTraits>;

  using RequestValidator_ = FileWriterRequestValidator;
  using ResponseValidator_ = FileWriterResponseValidator;
  enum MethodMinVersions : uint32_t {
    kWriteMinVersion = 0,
    kWriteStreamMinVersion = 0,
    kTruncateMinVersion = 0,
  };
  virtual ~FileWriter() {}


  using WriteCallback = base::OnceCallback<void(::base::File::Error, uint64_t)>;
  
  virtual void Write(uint64_t position, ::blink::mojom::blink::BlobPtr blob, WriteCallback callback) = 0;


  using WriteStreamCallback = base::OnceCallback<void(::base::File::Error, uint64_t)>;
  
  virtual void WriteStream(uint64_t position, mojo::ScopedDataPipeConsumerHandle stream, WriteStreamCallback callback) = 0;


  using TruncateCallback = base::OnceCallback<void(::base::File::Error)>;
  
  virtual void Truncate(uint64_t length, TruncateCallback callback) = 0;
};

class PLATFORM_EXPORT FileWriterProxy
    : public FileWriter {
 public:
  using InterfaceType = FileWriter;

  explicit FileWriterProxy(mojo::MessageReceiverWithResponder* receiver);
  void Write(uint64_t position, ::blink::mojom::blink::BlobPtr blob, WriteCallback callback) final;
  void WriteStream(uint64_t position, mojo::ScopedDataPipeConsumerHandle stream, WriteStreamCallback callback) final;
  void Truncate(uint64_t length, TruncateCallback callback) final;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class PLATFORM_EXPORT FileWriterStubDispatch {
 public:
  static bool Accept(FileWriter* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FileWriter* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FileWriter>>
class FileWriterStub
    : public mojo::MessageReceiverWithResponderStatus {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FileWriterStub() {}
  ~FileWriterStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FileWriterStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FileWriterStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class PLATFORM_EXPORT FileWriterRequestValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};
class PLATFORM_EXPORT FileWriterResponseValidator : public mojo::MessageReceiver {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_BLINK_PUBLIC_MOJOM_FILESYSTEM_FILE_WRITER_MOJOM_BLINK_H_