// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator.h"
#include "third_party/blink/renderer/platform/wtf/assertions.h"

namespace WTF {
class AtomicString;
class String;
}

namespace blink {

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kAnimationDelay = 2,
    kAnimationDirection = 3,
    kAnimationDuration = 4,
    kAnimationFillMode = 5,
    kAnimationIterationCount = 6,
    kAnimationName = 7,
    kAnimationPlayState = 8,
    kAnimationTimingFunction = 9,
    kTransitionDelay = 10,
    kTransitionDuration = 11,
    kTransitionProperty = 12,
    kTransitionTimingFunction = 13,
    kColor = 14,
    kDirection = 15,
    kFontFamily = 16,
    kFontFeatureSettings = 17,
    kFontKerning = 18,
    kFontSize = 19,
    kFontSizeAdjust = 20,
    kFontStretch = 21,
    kFontStyle = 22,
    kFontVariantCaps = 23,
    kFontVariantEastAsian = 24,
    kFontVariantLigatures = 25,
    kFontVariantNumeric = 26,
    kFontVariationSettings = 27,
    kFontWeight = 28,
    kTextOrientation = 29,
    kTextRendering = 30,
    kWebkitFontSmoothing = 31,
    kWebkitLocale = 32,
    kWebkitTextOrientation = 33,
    kWebkitWritingMode = 34,
    kWritingMode = 35,
    kZoom = 36,
    kAlignContent = 37,
    kAlignItems = 38,
    kAlignSelf = 39,
    kAlignmentBaseline = 40,
    kAll = 41,
    kBackdropFilter = 42,
    kBackfaceVisibility = 43,
    kBackgroundAttachment = 44,
    kBackgroundBlendMode = 45,
    kBackgroundClip = 46,
    kBackgroundColor = 47,
    kBackgroundImage = 48,
    kBackgroundOrigin = 49,
    kBackgroundPositionX = 50,
    kBackgroundPositionY = 51,
    kBackgroundRepeatX = 52,
    kBackgroundRepeatY = 53,
    kBackgroundSize = 54,
    kBaselineShift = 55,
    kBlockSize = 56,
    kBorderBlockEndColor = 57,
    kBorderBlockEndStyle = 58,
    kBorderBlockEndWidth = 59,
    kBorderBlockStartColor = 60,
    kBorderBlockStartStyle = 61,
    kBorderBlockStartWidth = 62,
    kBorderBottomColor = 63,
    kBorderBottomLeftRadius = 64,
    kBorderBottomRightRadius = 65,
    kBorderBottomStyle = 66,
    kBorderBottomWidth = 67,
    kBorderCollapse = 68,
    kBorderImageOutset = 69,
    kBorderImageRepeat = 70,
    kBorderImageSlice = 71,
    kBorderImageSource = 72,
    kBorderImageWidth = 73,
    kBorderInlineEndColor = 74,
    kBorderInlineEndStyle = 75,
    kBorderInlineEndWidth = 76,
    kBorderInlineStartColor = 77,
    kBorderInlineStartStyle = 78,
    kBorderInlineStartWidth = 79,
    kBorderLeftColor = 80,
    kBorderLeftStyle = 81,
    kBorderLeftWidth = 82,
    kBorderRightColor = 83,
    kBorderRightStyle = 84,
    kBorderRightWidth = 85,
    kBorderTopColor = 86,
    kBorderTopLeftRadius = 87,
    kBorderTopRightRadius = 88,
    kBorderTopStyle = 89,
    kBorderTopWidth = 90,
    kBottom = 91,
    kBoxShadow = 92,
    kBoxSizing = 93,
    kBreakAfter = 94,
    kBreakBefore = 95,
    kBreakInside = 96,
    kBufferedRendering = 97,
    kCaptionSide = 98,
    kCaretColor = 99,
    kClear = 100,
    kClip = 101,
    kClipPath = 102,
    kClipRule = 103,
    kColorInterpolation = 104,
    kColorInterpolationFilters = 105,
    kColorRendering = 106,
    kColumnCount = 107,
    kColumnFill = 108,
    kColumnGap = 109,
    kColumnRuleColor = 110,
    kColumnRuleStyle = 111,
    kColumnRuleWidth = 112,
    kColumnSpan = 113,
    kColumnWidth = 114,
    kContain = 115,
    kContent = 116,
    kCounterIncrement = 117,
    kCounterReset = 118,
    kCursor = 119,
    kCx = 120,
    kCy = 121,
    kD = 122,
    kDisplay = 123,
    kDominantBaseline = 124,
    kEmptyCells = 125,
    kFill = 126,
    kFillOpacity = 127,
    kFillRule = 128,
    kFilter = 129,
    kFlexBasis = 130,
    kFlexDirection = 131,
    kFlexGrow = 132,
    kFlexShrink = 133,
    kFlexWrap = 134,
    kFloat = 135,
    kFloodColor = 136,
    kFloodOpacity = 137,
    kFontDisplay = 138,
    kGridAutoColumns = 139,
    kGridAutoFlow = 140,
    kGridAutoRows = 141,
    kGridColumnEnd = 142,
    kGridColumnStart = 143,
    kGridRowEnd = 144,
    kGridRowStart = 145,
    kGridTemplateAreas = 146,
    kGridTemplateColumns = 147,
    kGridTemplateRows = 148,
    kHeight = 149,
    kHyphens = 150,
    kImageOrientation = 151,
    kImageRendering = 152,
    kInlineSize = 153,
    kInsetBlockEnd = 154,
    kInsetBlockStart = 155,
    kInsetInlineEnd = 156,
    kInsetInlineStart = 157,
    kIsolation = 158,
    kJustifyContent = 159,
    kJustifyItems = 160,
    kJustifySelf = 161,
    kLeft = 162,
    kLetterSpacing = 163,
    kLightingColor = 164,
    kLineBreak = 165,
    kLineHeight = 166,
    kLineHeightStep = 167,
    kListStyleImage = 168,
    kListStylePosition = 169,
    kListStyleType = 170,
    kMarginBlockEnd = 171,
    kMarginBlockStart = 172,
    kMarginBottom = 173,
    kMarginInlineEnd = 174,
    kMarginInlineStart = 175,
    kMarginLeft = 176,
    kMarginRight = 177,
    kMarginTop = 178,
    kMarkerEnd = 179,
    kMarkerMid = 180,
    kMarkerStart = 181,
    kMask = 182,
    kMaskSourceType = 183,
    kMaskType = 184,
    kMaxBlockSize = 185,
    kMaxHeight = 186,
    kMaxInlineSize = 187,
    kMaxWidth = 188,
    kMaxZoom = 189,
    kMinBlockSize = 190,
    kMinHeight = 191,
    kMinInlineSize = 192,
    kMinWidth = 193,
    kMinZoom = 194,
    kMixBlendMode = 195,
    kObjectFit = 196,
    kObjectPosition = 197,
    kOffsetAnchor = 198,
    kOffsetDistance = 199,
    kOffsetPath = 200,
    kOffsetPosition = 201,
    kOffsetRotate = 202,
    kOpacity = 203,
    kOrder = 204,
    kOrientation = 205,
    kOrphans = 206,
    kOutlineColor = 207,
    kOutlineOffset = 208,
    kOutlineStyle = 209,
    kOutlineWidth = 210,
    kOverflowAnchor = 211,
    kOverflowWrap = 212,
    kOverflowX = 213,
    kOverflowY = 214,
    kOverscrollBehaviorX = 215,
    kOverscrollBehaviorY = 216,
    kPaddingBlockEnd = 217,
    kPaddingBlockStart = 218,
    kPaddingBottom = 219,
    kPaddingInlineEnd = 220,
    kPaddingInlineStart = 221,
    kPaddingLeft = 222,
    kPaddingRight = 223,
    kPaddingTop = 224,
    kPage = 225,
    kPaintOrder = 226,
    kPerspective = 227,
    kPerspectiveOrigin = 228,
    kPointerEvents = 229,
    kPosition = 230,
    kQuotes = 231,
    kR = 232,
    kResize = 233,
    kRight = 234,
    kRotate = 235,
    kRowGap = 236,
    kRx = 237,
    kRy = 238,
    kScale = 239,
    kScrollBehavior = 240,
    kScrollCustomization = 241,
    kScrollMarginBlockEnd = 242,
    kScrollMarginBlockStart = 243,
    kScrollMarginBottom = 244,
    kScrollMarginInlineEnd = 245,
    kScrollMarginInlineStart = 246,
    kScrollMarginLeft = 247,
    kScrollMarginRight = 248,
    kScrollMarginTop = 249,
    kScrollPaddingBlockEnd = 250,
    kScrollPaddingBlockStart = 251,
    kScrollPaddingBottom = 252,
    kScrollPaddingInlineEnd = 253,
    kScrollPaddingInlineStart = 254,
    kScrollPaddingLeft = 255,
    kScrollPaddingRight = 256,
    kScrollPaddingTop = 257,
    kScrollSnapAlign = 258,
    kScrollSnapStop = 259,
    kScrollSnapType = 260,
    kShapeImageThreshold = 261,
    kShapeMargin = 262,
    kShapeOutside = 263,
    kShapeRendering = 264,
    kSize = 265,
    kSpeak = 266,
    kSrc = 267,
    kStopColor = 268,
    kStopOpacity = 269,
    kStroke = 270,
    kStrokeDasharray = 271,
    kStrokeDashoffset = 272,
    kStrokeLinecap = 273,
    kStrokeLinejoin = 274,
    kStrokeMiterlimit = 275,
    kStrokeOpacity = 276,
    kStrokeWidth = 277,
    kTabSize = 278,
    kTableLayout = 279,
    kTextAlign = 280,
    kTextAlignLast = 281,
    kTextAnchor = 282,
    kTextCombineUpright = 283,
    kTextDecorationColor = 284,
    kTextDecorationLine = 285,
    kTextDecorationSkipInk = 286,
    kTextDecorationStyle = 287,
    kTextIndent = 288,
    kTextJustify = 289,
    kTextOverflow = 290,
    kTextShadow = 291,
    kTextSizeAdjust = 292,
    kTextTransform = 293,
    kTextUnderlinePosition = 294,
    kTop = 295,
    kTouchAction = 296,
    kTransform = 297,
    kTransformBox = 298,
    kTransformOrigin = 299,
    kTransformStyle = 300,
    kTranslate = 301,
    kUnicodeBidi = 302,
    kUnicodeRange = 303,
    kUserSelect = 304,
    kUserZoom = 305,
    kVectorEffect = 306,
    kVerticalAlign = 307,
    kViewportFit = 308,
    kVisibility = 309,
    kWebkitAppRegion = 310,
    kWebkitAppearance = 311,
    kWebkitBorderHorizontalSpacing = 312,
    kWebkitBorderImage = 313,
    kWebkitBorderVerticalSpacing = 314,
    kWebkitBoxAlign = 315,
    kWebkitBoxDecorationBreak = 316,
    kWebkitBoxDirection = 317,
    kWebkitBoxFlex = 318,
    kWebkitBoxOrdinalGroup = 319,
    kWebkitBoxOrient = 320,
    kWebkitBoxPack = 321,
    kWebkitBoxReflect = 322,
    kWebkitFontSizeDelta = 323,
    kWebkitHighlight = 324,
    kWebkitHyphenateCharacter = 325,
    kWebkitLineBreak = 326,
    kWebkitLineClamp = 327,
    kWebkitMarginAfterCollapse = 328,
    kWebkitMarginBeforeCollapse = 329,
    kWebkitMarginBottomCollapse = 330,
    kWebkitMarginTopCollapse = 331,
    kWebkitMaskBoxImageOutset = 332,
    kWebkitMaskBoxImageRepeat = 333,
    kWebkitMaskBoxImageSlice = 334,
    kWebkitMaskBoxImageSource = 335,
    kWebkitMaskBoxImageWidth = 336,
    kWebkitMaskClip = 337,
    kWebkitMaskComposite = 338,
    kWebkitMaskImage = 339,
    kWebkitMaskOrigin = 340,
    kWebkitMaskPositionX = 341,
    kWebkitMaskPositionY = 342,
    kWebkitMaskRepeatX = 343,
    kWebkitMaskRepeatY = 344,
    kWebkitMaskSize = 345,
    kWebkitPerspectiveOriginX = 346,
    kWebkitPerspectiveOriginY = 347,
    kWebkitPrintColorAdjust = 348,
    kWebkitRtlOrdering = 349,
    kWebkitRubyPosition = 350,
    kWebkitTapHighlightColor = 351,
    kWebkitTextCombine = 352,
    kWebkitTextDecorationsInEffect = 353,
    kWebkitTextEmphasisColor = 354,
    kWebkitTextEmphasisPosition = 355,
    kWebkitTextEmphasisStyle = 356,
    kWebkitTextFillColor = 357,
    kWebkitTextSecurity = 358,
    kWebkitTextStrokeColor = 359,
    kWebkitTextStrokeWidth = 360,
    kWebkitTransformOriginX = 361,
    kWebkitTransformOriginY = 362,
    kWebkitTransformOriginZ = 363,
    kWebkitUserDrag = 364,
    kWebkitUserModify = 365,
    kWhiteSpace = 366,
    kWidows = 367,
    kWidth = 368,
    kWillChange = 369,
    kWordBreak = 370,
    kWordSpacing = 371,
    kX = 372,
    kY = 373,
    kZIndex = 374,
    kAnimation = 375,
    kBackground = 376,
    kBackgroundPosition = 377,
    kBackgroundRepeat = 378,
    kBorder = 379,
    kBorderBlock = 380,
    kBorderBlockColor = 381,
    kBorderBlockEnd = 382,
    kBorderBlockStart = 383,
    kBorderBlockStyle = 384,
    kBorderBlockWidth = 385,
    kBorderBottom = 386,
    kBorderColor = 387,
    kBorderImage = 388,
    kBorderInline = 389,
    kBorderInlineColor = 390,
    kBorderInlineEnd = 391,
    kBorderInlineStart = 392,
    kBorderInlineStyle = 393,
    kBorderInlineWidth = 394,
    kBorderLeft = 395,
    kBorderRadius = 396,
    kBorderRight = 397,
    kBorderSpacing = 398,
    kBorderStyle = 399,
    kBorderTop = 400,
    kBorderWidth = 401,
    kColumnRule = 402,
    kColumns = 403,
    kFlex = 404,
    kFlexFlow = 405,
    kFont = 406,
    kFontVariant = 407,
    kGap = 408,
    kGrid = 409,
    kGridArea = 410,
    kGridColumn = 411,
    kGridColumnGap = 412,
    kGridGap = 413,
    kGridRow = 414,
    kGridRowGap = 415,
    kGridTemplate = 416,
    kInset = 417,
    kInsetBlock = 418,
    kInsetInline = 419,
    kListStyle = 420,
    kMargin = 421,
    kMarginBlock = 422,
    kMarginInline = 423,
    kMarker = 424,
    kOffset = 425,
    kOutline = 426,
    kOverflow = 427,
    kOverscrollBehavior = 428,
    kPadding = 429,
    kPaddingBlock = 430,
    kPaddingInline = 431,
    kPageBreakAfter = 432,
    kPageBreakBefore = 433,
    kPageBreakInside = 434,
    kPlaceContent = 435,
    kPlaceItems = 436,
    kPlaceSelf = 437,
    kScrollMargin = 438,
    kScrollMarginBlock = 439,
    kScrollMarginInline = 440,
    kScrollPadding = 441,
    kScrollPaddingBlock = 442,
    kScrollPaddingInline = 443,
    kTextDecoration = 444,
    kTransition = 445,
    kWebkitColumnBreakAfter = 446,
    kWebkitColumnBreakBefore = 447,
    kWebkitColumnBreakInside = 448,
    kWebkitMarginCollapse = 449,
    kWebkitMask = 450,
    kWebkitMaskBoxImage = 451,
    kWebkitMaskPosition = 452,
    kWebkitMaskRepeat = 453,
    kWebkitTextEmphasis = 454,
    kWebkitTextStroke = 455,
    kAliasWebkitBorderEndColor = 586,
    kAliasWebkitBorderEndStyle = 587,
    kAliasWebkitBorderEndWidth = 588,
    kAliasWebkitBorderStartColor = 589,
    kAliasWebkitBorderStartStyle = 590,
    kAliasWebkitBorderStartWidth = 591,
    kAliasWebkitBorderBeforeColor = 572,
    kAliasWebkitBorderBeforeStyle = 573,
    kAliasWebkitBorderBeforeWidth = 574,
    kAliasWebkitBorderAfterColor = 569,
    kAliasWebkitBorderAfterStyle = 570,
    kAliasWebkitBorderAfterWidth = 571,
    kAliasWebkitMarginEnd = 686,
    kAliasWebkitMarginStart = 687,
    kAliasWebkitMarginBefore = 684,
    kAliasWebkitMarginAfter = 683,
    kAliasWebkitPaddingEnd = 732,
    kAliasWebkitPaddingStart = 733,
    kAliasWebkitPaddingBefore = 730,
    kAliasWebkitPaddingAfter = 729,
    kAliasWebkitLogicalWidth = 665,
    kAliasWebkitLogicalHeight = 568,
    kAliasWebkitMinLogicalWidth = 704,
    kAliasWebkitMinLogicalHeight = 702,
    kAliasWebkitMaxLogicalWidth = 699,
    kAliasWebkitMaxLogicalHeight = 697,
    kAliasWebkitBorderAfter = 894,
    kAliasWebkitBorderBefore = 895,
    kAliasWebkitBorderEnd = 903,
    kAliasWebkitBorderStart = 904,
    kAliasEpubCaptionSide = 610,
    kAliasEpubTextCombine = 864,
    kAliasEpubTextEmphasis = 966,
    kAliasEpubTextEmphasisColor = 866,
    kAliasEpubTextEmphasisStyle = 868,
    kAliasEpubTextOrientation = 545,
    kAliasEpubTextTransform = 805,
    kAliasEpubWordBreak = 882,
    kAliasEpubWritingMode = 546,
    kAliasWebkitAlignContent = 549,
    kAliasWebkitAlignItems = 550,
    kAliasWebkitAlignSelf = 551,
    kAliasWebkitAnimation = 887,
    kAliasWebkitAnimationDelay = 514,
    kAliasWebkitAnimationDirection = 515,
    kAliasWebkitAnimationDuration = 516,
    kAliasWebkitAnimationFillMode = 517,
    kAliasWebkitAnimationIterationCount = 518,
    kAliasWebkitAnimationName = 519,
    kAliasWebkitAnimationPlayState = 520,
    kAliasWebkitAnimationTimingFunction = 521,
    kAliasWebkitBackfaceVisibility = 555,
    kAliasWebkitBackgroundClip = 558,
    kAliasWebkitBackgroundOrigin = 561,
    kAliasWebkitBackgroundSize = 566,
    kAliasWebkitBorderBottomLeftRadius = 576,
    kAliasWebkitBorderBottomRightRadius = 577,
    kAliasWebkitBorderRadius = 908,
    kAliasWebkitBorderTopLeftRadius = 599,
    kAliasWebkitBorderTopRightRadius = 600,
    kAliasWebkitBoxShadow = 604,
    kAliasWebkitBoxSizing = 605,
    kAliasWebkitClipPath = 614,
    kAliasWebkitColumnCount = 619,
    kAliasWebkitColumnGap = 621,
    kAliasWebkitColumnRule = 914,
    kAliasWebkitColumnRuleColor = 622,
    kAliasWebkitColumnRuleStyle = 623,
    kAliasWebkitColumnRuleWidth = 624,
    kAliasWebkitColumnSpan = 625,
    kAliasWebkitColumnWidth = 626,
    kAliasWebkitColumns = 915,
    kAliasWebkitFilter = 641,
    kAliasWebkitFlex = 916,
    kAliasWebkitFlexBasis = 642,
    kAliasWebkitFlexDirection = 643,
    kAliasWebkitFlexFlow = 917,
    kAliasWebkitFlexGrow = 644,
    kAliasWebkitFlexShrink = 645,
    kAliasWebkitFlexWrap = 646,
    kAliasWebkitFontFeatureSettings = 529,
    kAliasWebkitJustifyContent = 671,
    kAliasWebkitOpacity = 715,
    kAliasWebkitOrder = 716,
    kAliasWebkitPerspective = 739,
    kAliasWebkitPerspectiveOrigin = 740,
    kAliasWebkitShapeImageThreshold = 773,
    kAliasWebkitShapeMargin = 774,
    kAliasWebkitShapeOutside = 775,
    kAliasWebkitTextSizeAdjust = 804,
    kAliasWebkitTransform = 809,
    kAliasWebkitTransformOrigin = 811,
    kAliasWebkitTransformStyle = 812,
    kAliasWebkitTransition = 957,
    kAliasWebkitTransitionDelay = 522,
    kAliasWebkitTransitionDuration = 523,
    kAliasWebkitTransitionProperty = 524,
    kAliasWebkitTransitionTimingFunction = 525,
    kAliasWebkitUserSelect = 816,
    kAliasWordWrap = 724,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
};

const int kIntFirstCSSProperty = 2;
const CSSPropertyID firstCSSProperty = static_cast<CSSPropertyID>(kIntFirstCSSProperty);
const int numCSSProperties = 554;
const int kIntLastCSSProperty = 455;
const CSSPropertyID lastCSSProperty = static_cast<CSSPropertyID>(kIntLastCSSProperty);
const CSSPropertyID lastUnresolvedCSSProperty = static_cast<CSSPropertyID>(966);
const int numCSSPropertyIDs = static_cast<int>(lastUnresolvedCSSProperty) + 1;
const size_t maxCSSPropertyNameLength = 40;

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, firstCSSProperty);
    DCHECK_LE(id, lastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

inline bool isCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= firstCSSProperty && id <= lastUnresolvedCSSProperty;
}

inline bool isValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID convertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

inline CSSPropertyID resolveCSSPropertyID(CSSPropertyID id)
{
    return convertToCSSPropertyID(static_cast<int>(id) & ~512);
}

inline bool isPropertyAlias(CSSPropertyID id) { return static_cast<int>(id) & 512; }

CSSPropertyID unresolvedCSSPropertyID(const WTF::String&);

CSSPropertyID CORE_EXPORT cssPropertyID(const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();
 public:
    class Iterator {
      STACK_ALLOCATED();
     public:
      Iterator(int id) : id_(id) {}
      CSSPropertyID operator*() const { return convertToCSSPropertyID(id_); }
      Iterator& operator++() { id_++; return *this; }
      bool operator!=(const Iterator& i) const { return id_ != i.id_; }
     private:
      int id_;
    };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
