// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_ACCESSIBILITY_MOJOM_AX_EVENT_MOJOM_H_
#define UI_ACCESSIBILITY_MOJOM_AX_EVENT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"

#include "mojo/public/cpp/bindings/mojo_buildflags.h"
#if BUILDFLAG(MOJO_TRACE_ENABLED)
#include "base/trace_event/trace_event.h"
#endif
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/accessibility/mojom/ax_event.mojom-shared.h"
#include "ui/accessibility/mojom/ax_event.mojom-forward.h"
#include "ui/accessibility/ax_enums.mojom-forward.h"
#include <string>
#include <vector>

#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"


#include "mojo/public/cpp/bindings/lib/native_enum_serialization.h"
#include "mojo/public/cpp/bindings/lib/native_struct_serialization.h"
#include "ui/accessibility/ax_event.h"




namespace ax {
namespace mojom {





class  AXEvent {
 public:
  using DataView = AXEventDataView;
  using Data_ = internal::AXEvent_Data;

  template <typename... Args>
  static AXEventPtr New(Args&&... args) {
    return AXEventPtr(
        base::in_place, std::forward<Args>(args)...);
  }

  template <typename U>
  static AXEventPtr From(const U& u) {
    return mojo::TypeConverter<AXEventPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AXEvent>::Convert(*this);
  }


  AXEvent();

  AXEvent(
      ::ax::mojom::Event event_type,
      int32_t id,
      ::ax::mojom::EventFrom event_from,
      int32_t action_request_id);

  ~AXEvent();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AXEventPtr>
  AXEventPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AXEvent>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::SerializeImpl<
        AXEvent::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static mojo::Message SerializeAsMessage(UserType* input) {
    return mojo::internal::SerializeAsMessageImpl<
        AXEvent::DataView>(input);
  }

  // The returned Message is serialized only if the message is moved
  // cross-process or cross-language. Otherwise if the message is Deserialized
  // as the same UserType |input| will just be moved to |output| in
  // DeserializeFromMessage.
  template <typename UserType>
  static mojo::Message WrapAsMessage(UserType input) {
    return mojo::Message(std::make_unique<
        internal::AXEvent_UnserializedMessageContext<
            UserType, AXEvent::DataView>>(0, 0, std::move(input)));
  }

  template <typename UserType>
  static bool Deserialize(const void* data,
                          size_t data_num_bytes,
                          UserType* output) {
    return mojo::internal::DeserializeImpl<AXEvent::DataView>(
        data, data_num_bytes, std::vector<mojo::ScopedHandle>(), output, Validate);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return AXEvent::Deserialize(
        input.size() == 0 ? nullptr : &input.front(), input.size(), output);
  }

  template <typename UserType>
  static bool DeserializeFromMessage(mojo::Message input,
                                     UserType* output) {
    auto context = input.TakeUnserializedContext<
        internal::AXEvent_UnserializedMessageContext<
            UserType, AXEvent::DataView>>();
    if (context) {
      *output = std::move(context->TakeData());
      return true;
    }
    input.SerializeIfNecessary();
    return mojo::internal::DeserializeImpl<AXEvent::DataView>(
        input.payload(), input.payload_num_bytes(),
        std::move(*input.mutable_handles()), output, Validate);
  }

  
  ::ax::mojom::Event event_type;
  
  int32_t id;
  
  ::ax::mojom::EventFrom event_from;
  
  int32_t action_request_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
AXEventPtr AXEvent::Clone() const {
  return New(
      mojo::Clone(event_type),
      mojo::Clone(id),
      mojo::Clone(event_from),
      mojo::Clone(action_request_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AXEvent>::value>::type*>
bool AXEvent::Equals(const T& other_struct) const {
  if (!mojo::Equals(this->event_type, other_struct.event_type))
    return false;
  if (!mojo::Equals(this->id, other_struct.id))
    return false;
  if (!mojo::Equals(this->event_from, other_struct.event_from))
    return false;
  if (!mojo::Equals(this->action_request_id, other_struct.action_request_id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ax

namespace mojo {


template <>
struct  StructTraits<::ax::mojom::AXEvent::DataView,
                                         ::ax::mojom::AXEventPtr> {
  static bool IsNull(const ::ax::mojom::AXEventPtr& input) { return !input; }
  static void SetToNull(::ax::mojom::AXEventPtr* output) { output->reset(); }

  static decltype(::ax::mojom::AXEvent::event_type) event_type(
      const ::ax::mojom::AXEventPtr& input) {
    return input->event_type;
  }

  static decltype(::ax::mojom::AXEvent::id) id(
      const ::ax::mojom::AXEventPtr& input) {
    return input->id;
  }

  static decltype(::ax::mojom::AXEvent::event_from) event_from(
      const ::ax::mojom::AXEventPtr& input) {
    return input->event_from;
  }

  static decltype(::ax::mojom::AXEvent::action_request_id) action_request_id(
      const ::ax::mojom::AXEventPtr& input) {
    return input->action_request_id;
  }

  static bool Read(::ax::mojom::AXEvent::DataView input, ::ax::mojom::AXEventPtr* output);
};

}  // namespace mojo

#endif  // UI_ACCESSIBILITY_MOJOM_AX_EVENT_MOJOM_H_